
# # # # # # # # # # # # # # # # # # # # # # # # # #
# Helpers: filename generation, SVN management... #
#                                                 #
# Jonas Sukys                                     #
# SAM, ETH Zurich, Switzerland                    #
# sukys.jonas@gmail.com                           #
# # # # # # # # # # # # # # # # # # # # # # # # # #

# === global imports

import os

# === local imports

from movies import *

# === configuration

log10_variance_cutoff = 1e-2

import warnings
warnings.filterwarnings("ignore", message="divide by zero encountered in log10")

# path for local saving of figures with descriptive TeX files
IMG_PATH = 'img'

# preset color list for plotting data from multiple experiments on the same subplot
colors = ['b', 'g', 'r', 'c', 'm', 'y', 'burlywood', 'chartreuse', 'k']

# preset marker list for plotting data from multiple experiments on the same subplot
markers = ['s', '+', 'x', 'o', '*', '|']

# default selection of colours and markers
select = {'MC':0, 'MLMC':1, 'MLMCb':2, 'MC2':3, 'MLMC2':4, 'MLMC2b':5, 'ALT1':6, 'ALT2':7, 'ALT3':8 }

# === classes

# provides an update'able progress bar for the command line
class Progress (object):
    
    def __init__ (self, prefix, steps, length=20):
      self.prefix = prefix
      self.length = length
      self.steps  = steps
      
      from sys import stdout
      self.stdout = stdout
    
    def update (self, step):
      text  = '\r' + self.prefix
      fraction = float(step) / self.steps
      text += '[' + '#'*int(round(fraction*self.length)) + ' '*int((self.length-round(fraction*self.length))) +']'
      text += ' ' + str(int(round(100*fraction))) + '%'
      self.stdout.write(text)
      self.stdout.flush()

# === routines

# initializes required figure size for subplots
def init_subplot(infolines=True):
  
  import pylab
  
  if infolines:
    pylab.figure(figsize=(2*8,6))
  else:
    pylab.figure(figsize=(2*8,5))

def apply_log (extent, d=None):
  
  import numpy
  
  if d.any():
    d = numpy.fabs(d)
  
  if d.any():
    d = numpy.log10(numpy.where(d < log10_variance_cutoff, log10_variance_cutoff, d))
  
  if extent:
    values_extent = [0,0]
    values_extent[0] = numpy.log10( numpy.maximum( log10_variance_cutoff, extent[0] ) )
    values_extent[1] = numpy.log10( numpy.maximum( log10_variance_cutoff, extent[1] ) )
    return [values_extent,d]
  else:
    return [None,d]

# formats title prefix
def get_title_prefix (self, stat, log=False):
  
  if log:
    if self.i.L != 0 or self.i.ML != 1:
      title_prefix = 'log10(' + stat + ") of "
    else:
      title_prefix = 'log10 of '
  
  else:
    if self.i.L != 0 or self.i.ML != 1:
      title_prefix = stat + " of "
    else:
      title_prefix = ""
  
  return title_prefix

# generates filename including equation type, model and (possibly) flux, material or topography
def getFileName (refr, suffix=None, file_suffix='pdf', stat=None):

  if not os.path.exists(IMG_PATH):
    os.mkdir(IMG_PATH)
    os.system('svn add ' + IMG_PATH)
  if 'equation' in refr.opts:
    name = os.path.join(IMG_PATH,refr.opts['equation'])
    name += '-' + refr.opts['model']
  else:
    name = os.path.join(IMG_PATH,refr.opts['model'])
  if 'equation' in refr.opts and refr.opts['equation'] == 'sw':
    name += '-' + refr.opts['topography']
  if 'flux' in refr.opts and refr.opts['flux'] != 'det':
    name += '-' + refr.opts['flux']
  if 'equation' in refr.opts and refr.opts['equation'] == 'wave':
    name += '-' + refr.opts['material']
  if suffix:
    name += '_' + suffix
  if stat:
    name += '-' + stat
  if file_suffix:
    name += '.' + file_suffix
  return name

# returns values of coordinates of the cell centers in direction 'dir'
def get_range (self, dir):
  
  import numpy
  if dir == 'X':
    return numpy.arange(self.nx)*(self.f.MAXX/self.nx)+0.5*self.f.MAXX/self.nx
  elif dir == 'Y':
    return numpy.arange(self.ny)*(self.f.MAXY/self.ny)+0.5*self.f.MAXY/self.ny
  else:
    numpy.arange(self.nz)*(self.f.MAXZ/self.nz)+0.5*self.f.MAXZ/self.nz

# define needed functional (basen on kernel 'phi')
def evaluate_functional (d, r, phi):
  xr = get_range(r,'X')
  import numpy
  Phi = numpy.sum(d*phi(xr))
  Phi /= r.i.NX * r.i.MAXX
  return Phi

# updates SVN
def svnup():
  if not os.path.exists(IMG_PATH):
    os.system('svn up')

# quietly adds newly generated files to SVN
def svnadd():
  os.system('svn add -q ' + os.path.join(IMG_PATH, '*'))
