
# # # # # # # # # # # # # # # # # # # # # # # # # #
# Movie generation routines                       #
#                                                 #
# Jonas Sukys                                     #
# SAM, ETH Zurich, Switzerland                    #
# sukys.jonas@gmail.com                           #
# # # # # # # # # # # # # # # # # # # # # # # # # #

# === local imports

from helpers import *

# === routines

def getExtents (self, v, last_extent, topography=False, margin=0.0, level=None, type=None):
  
  ext_mean = getExtent (self, v, "mean", last_extent, topography, margin, level, type)
  
  if self.VARIANCE:
    ext_var = getExtent (self, v, "variance", last_extent, topography, margin, level, type)
    return [ext_mean, ext_var]
  else:
    return [ext_mean, [0,0]]

def getExtent (self, v, stat, last_extent, topography=False, margin=0.0, level=None, type=None):
  
  if last_extent:
    extent = getExtent_ts (self, v, len(self.times) - 1, stat, topography, margin, level, type)
  else:
    extent = getExtent_ts (self, v, 0, stat, topography, margin, level, type)
    
    for ts in range(1,len(self.times)):
      extent_tmp = getExtent_ts (self, v, ts, stat, topography, margin, level, type)
      extent = [min(extent_tmp[0], extent[0]), max(extent_tmp[1], extent[1])]
  
  return extent

def getExtent_ts (self, v, ts, stat, topography, margin, level, type):
  
  # TODO: implement topography
  
  import numpy
  
  d = self.var(v, ts, stat, level=level, type=type)
  
  low  = numpy.min(d)
  high = numpy.max(d)
  diff = high - low
  
  low  -= diff * margin
  high += diff * margin
  
  return [low, high]

def generate_PDF_movie (self, pdf, pylab, still_frames):
  
  print
  
  print ' :: Multiplying last frame...',
  # copy last frame several times
  for ts in range(len(self.times), len(self.times) + still_frames):
    pdf.savefig()
  pylab.close()
  print ' done.'
  
  # set the metadata of the PDF
  d = pdf.infodict()
  d['Title'] = 'Snapshots of the (MLMC-)FVM simulation using ALSVID-UQ software'
  d['Author'] = 'Jonas Sukys'
  d['Subject'] = 'Snapshots of the (MLMC-)FVM simulation using ALSVID-UQ software'
  d['Keywords'] = 'ALSVID-UQ MLMC-FVM movie'
  import datetime
  d['CreationDate'] = datetime.datetime.today()
  d['ModDate'] = datetime.datetime.today()
  
  print ' :: Converting into PDF...',
  # close PDF
  pdf.close()
  print ' done.'

def generate_movies (self, frames, save, suffix, default_file, output_formats, duration, still_frames):
  
  print
  print
  
  import os
  
  # copy last frame several times
  if still_frames:
    
    print ' :: Multiplying last frame...',
    for ts in range(frames, frames + still_frames):
      os.system( 'cp frame_%04d.png frame_%04d.png' % ( frames - 1, ts ) )
    print ' done.'
  
  # get file name
  if save:
    file_name = save
    if suffix:
      file_name += '-' + suffix
  else:
    file_name = getFileName (self, default_file + '-' + v.name, file_suffix=None, stat=suffix)
  
  # output PDF file with multiple pages
  if 'pdf' in output_formats:
    
    print ' :: Converting into PDF...',
    os.system('convert frame_*.png %s.pdf' % file_name )
    print ' done.'
  
  # encode mp4 movie from PNG snapshots
  if 'mp4' in output_formats:
    
    if not duration: duration = float(frames) / 25
    
    fps = float(frames) / duration
    
    print " :: Frame rate: " + str(fps)
    if fps < 25:
      print "  : -> consider increasing parameter 'NSAVES' to increase the frame rate!"
    
    print ' :: Converting into MP4...',
    os.system('mencoder "mf://*.png" -mf fps=%d -o %s.mp4 -ovc lavc -lavcopts vcodec=mpeg4:vbitrate=800 >> /dev/null &> /dev/null' % (fps, file_name))
    print ' done.'
    print
  
  os.system('rm frame_*.png')
