
# # # # # # # # # # # # # # # # # # # # # # # # # #
# Plotting routines for 3d simulation data        #
# Paraview backend                                #
#                                                 #
# Jonas Sukys                                     #
# SAM, ETH Zurich, Switzerland                    #
# sukys.jonas@gmail.com                           #
# # # # # # # # # # # # # # # # # # # # # # # # # #

# === global imports

import numpy

import paraview.simple as pv

from vtk.util import numpy_support as npvtk

# === local imports

from variables import *
from helpers import *

# === configuration

# === plotting routines

def get_view (view=None):
  
  current_view = {}
  
  current_view['azimuth']   = mlab.view()[0]
  current_view['elevation'] = mlab.view()[1]
  current_view['distance']  = mlab.view()[2]
  current_view['yaw']       = mlab.yaw()
  current_view['roll']      = mlab.roll()
  current_view['pitch']     = mlab.pitch()
  
  if view:
    if 'azimuth'   in view and view['azimuth']   != None : current_view['azimuth']   = view['azimuth']
    if 'elevation' in view and view['elevation'] != None : current_view['elevation'] = view['elevation']
    if 'distance'  in view and view['distance']  != None : current_view['distance']  = view['distance']
    if 'yaw'       in view and view['yaw']       != None : current_view['yaw']       = view['yaw']
    if 'roll'      in view and view['roll']      != None : current_view['roll']      = view['roll']
    if 'pitch'     in view and view['pitch']     != None : current_view['pitch']     = view['pitch']
  
  return current_view

def set_view (view):
  
  mlab.view ( azimuth=view['azimuth'], elevation=view['elevation'], distance=view['distance'] )
  mlab.yaw   (view['yaw'])
  mlab.roll  (view['roll'])
  mlab.pitch (view['pitch'])

def plot_3d_stat (self, v, ts, stat, extent, infolines, view, save, log, contour, num_contours, contour_levels, clip, highlight, level, type, movie=False, engine=None):
  
  # routines from examplt script:
  
  view = GetRenderView()
  
  sphere = Sphere()
  sphere.ThetaResolution = 100
  sphere.PhiResolution = 100
  
  pidscal = ProcessIdScalars(sphere)
  
  rep = Show(pidscal)
  
  nbprocs = servermanager.ActiveConnection.GetNumberOfDataPartitions()
  drange = [0, nbprocs-1]
  
  lt = MakeBlueToRedLT(drange[0], drange[1])
  lt.NumberOfTableValues = nbprocs
  
  rep.LookupTable = lt
  rep.ColorAttributeType = 'POINT_DATA'
  rep.ColorArrayName = "ProcessId"
  
  bar = CreateScalarBar(LookupTable=lt, Title="PID")
  bar.TitleColor = [0,0,0]
  bar.LabelColor = [0,0,0]
  bar.NumberOfLabels = 6
  view.Representations.append(bar)
  
  view.Background = [.7, .7, .7]
  view.CameraViewUp = [0, 1, 0]
  view.StillRender()
  view.ResetCamera()
  # change the pathname to a place where you have write access
  WriteImage("/project/csvis/ParaViewTests/coloredSphere.00.png",  Writer="vtkPNGWriter")
  
  view.CameraViewUp = [0, 0, 1]
  view.CameraFocalPoint = [0, 0, 0]
  view.CameraPosition = [0, 1, 0]
  view.ResetCamera()
  # change the pathname to a place where you have write access
  WriteImage("/project/csvis/ParaViewTests/coloredSphere.01.png",  Writer="vtkPNGWriter")
  
  view.CameraViewUp = [0, 0, 1]
  view.CameraFocalPoint = [0, 0, 0]
  view.CameraPosition = [0, -1, 0]
  view.ResetCamera()
  # change the pathname to a place where you have write access
  WriteImage("/project/csvis/ParaViewTests/coloredSphere.02.png",  Writer="vtkPNGWriter")
  
  # END OF example routines
  
  d = self.var(v, ts, stat)
  if log:
    [values_extent,d] = apply_log (extent,d)
  else:
    values_extent = extent
  
  if values_extent:
    vmin = values_extent[0]
    vmax = values_extent[1]
  else:
    vmin = None
    vmax = None
  
  title_prefix = get_title_prefix (self, stat, log)
  
  [xr, yr, zr] = numpy.mgrid[0:self.nx, 0:self.ny, 0:self.nz]
  xr = xr * self.f.MAXX / self.nx
  yr = yr * self.f.MAXY / self.ny
  zr = zr * self.f.MAXZ / self.nz
  
  # convert numpy to vtk array
  d_vtk = npvtk.numpy_to_vtk(d)
  
  # TODO: from here!
  
  #if not movie: mlab.figure(size=(800, 800))
  
  pv.Connect()
  
  if contour:
    mlab.contour3d(xr, yr, zr, d_vtk, contours=num_contours, opacity=0.5, vmin=vmin, vmax=vmax, name=title_prefix+v.name)
  else:
    mlab.pipeline.volume(mlab.pipeline.scalar_field(d_vtk), vmin=vmin, vmax=vmax, name=title_prefix+v.name)
  
  mlab.title(title_prefix + v.name + ' at t=%.2f' % self.times[ts], height=0.95, size=0.3, color=black)
  
  mlab.axes ( x_axis_visibility=False, y_axis_visibility=False, z_axis_visibility=False )
  ''' BUGGY! WHY?
  if not movie:
    mlab.axes ( color=black, extent=(0, self.f.MAXX, 0, self.f.MAXY, 0, self.f.MAXZ ) )
    mlab.xlabel('x')
    mlab.ylabel('y')
    mlab.zlabel('z')
  else:
    mlab.axes ( x_axis_visibility=False, y_axis_visibility=False, z_axis_visibility=False )
  '''
  
  mlab.colorbar()
  
  pv.bar = CreateScalarBar(LookupTable=lt, Title=self.name)
  #pv.bar.TitleColor = [0,0,0]
  #pv.bar.LabelColor = [0,0,0]
  #pv.bar.NumberOfLabels = 6
  pv.view.Representations.append(bar)
  
  if view:
    if 'azimuth'   in view and view['azimuth']   : mlab.view  (azimuth   = view['azimuth'])
    if 'elevation' in view and view['elevation'] : mlab.view  (elevation = view['elevation'])
    if 'distance'  in view and view['distance']  : mlab.view  (distance  = view['distance'])
    if 'yaw'       in view and view['yaw']       : mlab.yaw   (view['yaw'])
    if 'roll'      in view and view['roll']      : mlab.roll  (view['roll'])
    if 'pitch'     in view and view['pitch']     : mlab.pitch (view['pitch'])
  
  if infolines: show_info(self, level, type)
  
  if save: pv.WriteImage(save[0:-4] + '-' + stat + save[len(save)-4:len(save)],  Writer="vtkPNGWriter", size=(800,800))
  
  if not movie: Render()

def plot_3d (self, v, ts, extent_mean, extent_var, infolines, view, save, log_mean, log_var, contour, num_contours, contour_levels, clip, highlight, level, type, engine):
  
  if ts is None: ts = len(self.times) - 1
  
  # plot mean
  plot_3d_stat (self, v, ts, "mean", extent_mean, infolines, view, save, log_mean, contour, num_contours, contour_levels, clip, highlight, level, type, engine)
  
  # plot variance
  if self.VARIANCE:
    plot_3d_stat (self, v, ts, "variance", extent_var, infolines, view, save, log_var, contour, num_contours, contour_levels, clip, highlight, level, type, engine)
  
  # finalization
  if save:
    self.generateTexTable(save)
    self.generateTexTableVertical(save)

# === movie generation

def movie_3d (self, v, duration, infolines, last_extent, output_formats, still_frames, view, log_mean, log_var, contour, num_contours, contour_levels, clip, highlight, level, type, save, engine):
  
  # TODO

def movie_rotate_3d (self, v, ts, duration, frames, infolines, output_formats, still_frames, view, log_mean, log_var, contour, num_contours, contour_levels, clip, highlight, level, type, save, engine):
  
  # TODO

# === helpers

def show_info(self, level, type):
      
  text = self.generateInfo(prefix='', level=level, type=type)
  
  mlab.text(0.01, 0.01, text, color=black, line_width=2.0, width=1.0, name='infolines')
