
# # # # # # # # # # # # # # # # # # # # # # # # # #
# Wrapping routines                               #
# for 3-dimensional simulation data               #
#                                                 #
# Jonas Sukys                                     #
# SAM, ETH Zurich, Switzerland                    #
# sukys.jonas@gmail.com                           #
# # # # # # # # # # # # # # # # # # # # # # # # # #

# === local imports

from helpers import *

# write BOV ("blocks of values") header
def write_BOV (self, v, ts, stat, level, type):
  
  variable = get_title_prefix (self, stat) + v.name
  
  filename_mean = self.outputf
  filename_var  = self.varf
  if level != None and type != None:
    filename_mean += '_MESH_LEVEL=' + str(level) + '_TYPE=' + str(type)
    filename_var  += '_MESH_LEVEL=' + str(level) + '_TYPE=' + str(type)
  
  split = False if os.path.exists(self.outputf) else True
  
  text = ''
  
  text += 'TIME: %f' % self.times[ts] + '\n'
  
  if stat == 'mean':
    if split:
      text += 'DATA_FILE: %s-%d' % (filename_mean, ts) + '\n'
    else:
      text += 'DATA_FILE: %s' % filename_mean + '\n'
  else:
    if split:
      text += 'DATA_FILE: %s-%d' % (filename_var, ts) + '\n'
    else:
      text += 'DATA_FILE: %s' % filename_var + '\n'
  
  text += 'DATA_SIZE: %d %d %d' % (self.i.NX, self.i.NY, self.i.NZ) + '\n'
  
  text += 'DATA_FORMAT: DOUBLE' + '\n'
  
  text += 'VARIABLE: "%s"' % variable + '\n'
  
  text += 'DATA_ENDIAN: LITTLE' + '\n'
  
  text += 'CENTERING: nodal' + '\n'
  
  text += 'BRICK_SIZE: %f %f %f' % (self.f.MAXX, self.f.MAXY, self.f.MAXZ) + '\n'
  
  text += 'DATA_SIZE: %d %d %d' % (self.i.NX, self.i.NY, self.i.NZ) + '\n'
  
  if split:
    text += 'BYTE_OFFSET: %d' % ( 8 * self.i.NX * self.i.NY * self.i.NZ * v.i ) + '\n'
  else:
    text += 'BYTE_OFFSET: %d' % ( 8 * self.i.NX * self.i.NY * self.i.NZ * (v.i + self.NVARS*ts) ) + '\n'
  
  text += 'DATA_COMPONENTS: 1' + '\n'
  
  # parallelization setup
  
  MULTIX = 2
  MULTIY = 2
  MULTIZ = 2
  
  text += 'DIVIDE_BRICK: true' + '\n'
  
  text += 'DATA_BRICKLETS: %d %d %d' % (self.i.NX / MULTIX, self.i.NY / MULTIY, self.i.NZ / MULTIZ) + '\n'
  
  # saving
  
  bov_file_name = variable + " at time " + str(self.times[ts])
  f = open(bov_file_name, 'w')
  f.write(text)
  f.close()
  
  return bov_file_name, variable

# delete BOV header
def delete_BOV (bov_file_name):
  import os
  os.system ('rm ' + '"' + bov_file_name + '"')

# write wrappers for each stat and time snapshot of variable 'v'
def wrap (self, v):
  
  for ts in range(self.i.SAVE0 + self.i.NSAVES):
    write_BOV (self, v, ts, 'mean')
  
  if r.i.STATS_COUNT == 2:
    for ts in range(self.i.SAVE0 + self.i.NSAVES):
      write_BOV (self, v, ts, 'variance')
