
#include <cstdlib>
#include <iostream>

#include "chaos.h"
#include "rng.h"

int CHAOS_SIZE [CHAOS_STREAMS] [CHAOS_TYPES] [CHAOS_DISTRIBUTIONS] = {0};

std::vector <real> chaos [CHAOS_STREAMS] [CHAOS_TYPES] [CHAOS_DISTRIBUTIONS];

// last offset used in 'chaos_init()'
static long_int last_offset;

// index (1, 2, ...) of the NEXT draw, AFTER the last 'chaos_init()'
static long_int next_index;

// buffers for routine 'draw_NORMAL()'
static real NORMAL_buffer;
static bool NORMAL_buffer_empty;

// helper variables
real U1, U2, r, w, N1, N2;
int stream, distribution, type;

// generates standard normal random variable using rejection method and Box-Muller transformation
real draw_NORMAL () {
  
  if (!NORMAL_buffer_empty) {
    
    NORMAL_buffer_empty = true;
    return NORMAL_buffer;
  }
  
  else {
    
    // rejection method to obtain (U1,U2) - uniform random variable on unit circle
    do {
      // draw two uniform random variable on [-1, 1]
      U1 = 2*draw_UNIFORM() - 1;
      U2 = 2*draw_UNIFORM() - 1;
      // compute radius
      r = sqr(U1) + sqr(U2);
    } while (r > 1.0);
    
    // Box-Muller transformation
    w  = sqrt(-2*log(r)/r);
    N1 = U1 * w;
    N2 = U2 * w;
    
    NORMAL_buffer = N2;
    NORMAL_buffer_empty = false;
    
    return N1;
  }
}

void chaos_draw () {
  
  for ( stream = 0; stream < CHAOS_STREAMS; stream++ )
    
    for ( type = 0; type < CHAOS_TYPES; type++ ) {
      
      for ( int i = 0; i < CHAOS_SIZE[stream][type][UNIFORM]; i++ )
        chaos [stream] [type] [UNIFORM] [i] = draw_UNIFORM();
      
      for ( int i = 0; i < CHAOS_SIZE[stream][type][NORMAL];  i++ )
        chaos [stream] [type] [NORMAL]  [i] = draw_NORMAL();
    }
  
  for ( int i = 0; i < CHAOS_SIZE[FLUX][GLOBAL][NORMAL]; i++ )
    
    if (fabs(chaos [FLUX] [GLOBAL] [NORMAL] [i]) > 1e3)
      
      std::cout << chaos [FLUX] [GLOBAL] [NORMAL] [i] << std::endl;
  
  next_index++;
}

void chaos_draw (long_int index) {
  chaos_shift (index);
  chaos_draw ();
}

void chaos_init (long_int offset) {
  
  // resizing vectors
  for ( stream = 0; stream < CHAOS_STREAMS; stream++ )
    for ( type = 0; type < CHAOS_TYPES; type++ )
      for ( distribution = 0; distribution < CHAOS_DISTRIBUTIONS; distribution++ )
        chaos [stream] [type] [distribution] .resize ( CHAOS_SIZE [stream] [type] [distribution] );
  
  // offsetting samples
  if ( offset > 0 )
    for ( long_int i = 1; i <= offset; i++)
      chaos_draw ();
  
  // reseting 'next_index'
  next_index = 1;
  
  // storing offset value
  last_offset = offset;
  
  // clear buffer for normal RV
  NORMAL_buffer_empty = true;
}

void chaos_reinit () {
  chaos_init (last_offset);
}

void chaos_shift (long_int index) {
  // if later index is required: simply offset the difference
  if ( index > next_index )
    // offsetting samples
    for (long_int i=next_index; i<index; i++)
      chaos_draw ();
  // if earlier index is required: reinit rng and chaos, then offset the difference
  else if ( index < next_index ) {
    rng_reinit();
    chaos_reinit();
    chaos_shift (index);
  }
}
