
#ifndef CHAOS_H
#define CHAOS_H

#include "numtypes.h"

#include <vector>

// independent streams for random numbers
const int CHAOS_STREAMS = 3;
const int MODEL      = 0;   // random numbers for initial data
const int FLUX       = 1;   // random numbers for fluxes
const int TOPOGRAPHY = 2;   // random numbers for bottom topography

// some random numbers might depend on domain index, and some might not
const int CHAOS_TYPES = 2;
const int GLOBAL = 0;      // depend on sample index only
const int LOCAL  = 1;      // depend on sample index and domain index

// distributions of random numbers within each stream
const int CHAOS_DISTRIBUTIONS = 2;
const int UNIFORM = 0;      // uniform distribution U[0,1]
const int NORMAL  = 1;      // normal distribution N[0,1]

// amount of random numbers within each stream and distribution
extern int CHAOS_SIZE [CHAOS_STREAMS] [CHAOS_TYPES] [CHAOS_DISTRIBUTIONS];

// array of vectors for random numbers
extern std::vector <real> chaos [CHAOS_STREAMS] [CHAOS_TYPES] [CHAOS_DISTRIBUTIONS];

// initializes iid array
void chaos_init (long_int offset=0);

// re-initializes iid array from the last offset
void chaos_reinit ();

// shifts the iid array to the last offset + index
void chaos_shift (long_int index);

// creates ONE draw of the random input variables (no index specified)
void chaos_draw ();

// creates ONE draw of the random input variables at index 'index'
void chaos_draw (long_int index);

#endif
