
/* EQUATION SPECIFIC FVM routines
 * Burgers' (scalar nonlinear conservation law)
 * 
 * For detailed description refer to:
 * 
 * S. Mishra and N. H. Risebro and C. Schwab and S. Tokareva
 * Numerical solution of scalar conservation laws with random flux functions.
 * In review, SAM Report 2012/35.
 * Available at: http://www.sam.math.ethz.ch/reports/2012/35
 */

///::title:: Burgers' (scalar nonlinear conservation law)
///::vars:: VISCOSITY=0
///::macro:: VISCOSITY_HOOK @ Source[i][j][k] += VISCOSITY * (U[i-1][j][k].u - 2*U[i][j][k].u + U[i+1][j][k].u) / (dx[X]*dx[X]);

#include "equation.h"

void set_chaos_size () {
  model_set_chaos_size();
  flux_set_chaos_size();
}

void apply_constraints (ConservedVars &CV, XtraVars &XV)
{
  // Constraints for ConservedVars
  /* nothing */
  // Constraints for XtraVars
  if (XV.E < 0.0) XV.E = 0.0;
}

void time_step_extend_coefficients () {}

#if INDICATOR == 1
// u
real indicator (const int &i, const int &j, const int &k, const int &d) {
  ConservedVars &Uijk = U[i][j][k];
  return Uijk.u;
}
#endif
