
/* EQUATION SPECIFIC FVM routines
 * Euler (Euler equations of gas dynamics)
 * 
 * For detailed description, refer to:
 * 
 * S. Mishra, Ch. Schwab and J. Šukys
 * Multi-level Monte Carlo finite volume methods for nonlinear systems of conservation laws in multi-dimensions.
 * Journal of Computational Physics, 231(8):3365--3388, 2012.
 * DOI: 10.1016/j.jcp.2012.01.011
 */

///::title:: Euler (Euler equations of gas dynamics)

#include "equation.h"

void set_chaos_size () {
  model_set_chaos_size();
  flux_set_chaos_size();
}

void apply_constraints (ConservedVars &CV, XtraVars &XV)
{
  // Constraints for ConservedVars
  if (CV.rho < 0.0) CV.rho = 0.0;
  if (CV.E   < 0.0) CV.E   = 0.0;
  // Constraints for XtraVars
  if (XV.p   < 0.0) XV.p   = 0.0;
}

void time_step_extend_coefficients () {}

#if INDICATOR == 1
// rho
real indicator (const int &i, const int &j, const int &k, const int &d) {
  ConservedVars &Uijk = U[i][j][k];
  return Uijk.rho;
}
#endif
