
/* EQUATION SPECIFIC FVM routines
 * Shallow Water equations
 * 
 * For detailed description, refer to:
 * 
 * S. Mishra, Ch. Schwab and J. Šukys
 * Multi-level Monte Carlo Finite Volume methods for shallow water equations with uncertain topography in multi-dimensions.
 * SIAM J. Sci. Comput., 34(6), pp. B761--B784, 2012.
 * DOI: 10.1137/110857295
 */

///::title:: Shallow Water Equations
///::vars:: GRAVITY=9.812 BED_OFFSET=0.0 BED_SCALING=1.0 CFL=0.45
///::depends:: topography
///::macro:: INITIAL_DATA_HOOK @ P.h -= Xtra[i][j][0].b; if (P.h < 0.0) P.h = 0.0;
///::macro:: SOURCE_HOOK topography.h compute_source(i,j);

#include "equation.h"
#include "solver-util-ec.h"

void set_chaos_size () {
  model_set_chaos_size();
  flux_set_chaos_size();
  topography_set_chaos_size();
}

void apply_constraints (ConservedVars &CV, XtraVars &XV)
{
  // Constraints for ConservedVars
  if (CV.h < 0.0) CV.h = 0.0;
  // Constraints for XtraVars
  if (XV.E < 0.0) XV.E = 0.0;
  if (XV.S < XV.b) XV.S = XV.b;
}

void time_step_extend_coefficients () {}

#if INDICATOR == 1
real indicator (const int &i, const int &j, const int &k, const int &d) {
  ConservedVars &Uijk = U[i][j][k];
  return Uijk.h;
}
#endif
