
/* EQUATION SPECIFIC FVM routines
 * Wave (Wave equation, 2nd type)
 * 
 * For detailed description, refer to:
 * 
 * J. Šukys, Ch. Schwab and S. Mishra
 * Multi-Level Monte Carlo Finite Difference and Finite Volume methods for stochastic linear hyperbolic systems.
 * MCQMC 2012 Springer Proceedings in Mathematics \& Statistics, Volume 65, pp. 649--666, 2013.
 * DOI: 10.1007/978-3-642-41095-6_34
 */

///::title:: Wave (Wave equation)
///::consts:: ISOTROPIC=1
///::vars:: MATERIAL_OFFSET=0.0 MATERIAL_SCALING=1.0
///::depends:: material

#include "equation.h"

void set_chaos_size () {
  model_set_chaos_size();
  flux_set_chaos_size();
}

void apply_constraints (ConservedVars &CV, XtraVars &XV) {
  // Constraints for ConservedVars
  // Constraints for XtraVars
  if (XV.E < 0.0) XV.E = 0.0;
  if (XV.c < 0.0) XV.c = 0.0;
}

void time_step_extend_coefficients () {
  
  // do constant extrapolation, i.e. copy from the closest cell
  
  #if NX != 1
  
  if (!multi_bc[X][HEAD])
    for (int i = 0; i < GX; i++)
      for (int j = 0; j < NYl; j++)
        for (int k = 0; k < NZl; k++)
          Xtra [-1-i][j][k] .c = Xtra [0][j][k] .c;
  
  if (!multi_bc[X][TAIL])
    for (int i = 0; i < GX; i++)
      for (int j = 0; j < NYl; j++)
        for (int k = 0; k < NZl; k++)
          Xtra [i+NXl][j][k] .c = Xtra [NXl-1][j][k] .c;
  
  #endif
  
  #if NY != 1
  
  if (!multi_bc[Y][HEAD])
    for (int i = 0; i < NXl; i++)
      for (int j = 0; j < GY; j++)
        for (int k = 0; k < NZl; k++)
          Xtra [i][-1-j][k] .c = Xtra [i][0][k] .c;
  
  if (!multi_bc[Y][TAIL])
    for (int i = 0; i < NXl; i++)
      for (int j = 0; j < GY; j++)
        for (int k = 0; k < NZl; k++)
          Xtra [i][j+NYl][k] .c = Xtra [i][NYl-1][k] .c;
  
  #endif
  
  #if NZ != 1
  
  if (!multi_bc[Z][HEAD])
    for (int i = 0; i < NXl; i++)
      for (int j = 0; j < NYl; j++)
        for (int k = 0; k < GZ; k++)
          Xtra [i][j][-1-k] .c = Xtra [i][j][0] .c;
  
  if (!multi_bc[Z][TAIL])
    for (int i = 0; i < NXl; i++)
      for (int j = 0; j < NYl; j++)
        for (int k = 0; k < GZ; k++)
          Xtra [i][j][k+NZl] .c = Xtra [i][j][NZl-1] .c;
  
  #endif
}

#if INDICATOR == 1
real indicator (const int &i, const int &j, const int &k, const int &d) {
  ConservedVars &Uijk = U[i][j][k];
  return Uijk.p;
}
#endif
