
/* EQUATION SPECIFIC FVM routines for FLUX
 * Flux with perturbed permeabilities (legacy)
 */

///::title:: Flux with perturbed permeabilities (legacy)
///::vars:: OIL_VISCOSITY=1.0 WATER_VISCOSITY=1.0
///::consts:: PERM_PERT_OIL=0.3 PERM_PERT_WATER=0.2

#include "flux.h"
#include "chaos.h"

void flux_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = 2;
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL]  = 0;
}

// relative oil permeability
real perm_oil (const ConservedVars &CV, const XtraVars &XV) {
  return sqr(CV.So) + PERM_PERT_OIL*(2 * chaos [FLUX] [GLOBAL] [UNIFORM] [0] - 1)*sqr(CV.So)*(1.0 - CV.So);
}

// relative water permeability
real perm_water (const ConservedVars &CV, const XtraVars &XV) {
  return sqr(XV.Sw) + PERM_PERT_WATER*(2 * chaos [FLUX] [GLOBAL] [UNIFORM] [1] - 1)*sqr(XV.Sw)*(1.0 - XV.Sw);;
}

// derivative of relative oil permeability
real d_dSo_perm_oil (const ConservedVars &CV, const XtraVars &XV) {
  return 2*CV.So + PERM_PERT_OIL*(2 * chaos [FLUX] [GLOBAL] [UNIFORM] [0] - 1) * CV.So * (2.0 - 3.0*CV.So);
}

// derivative water relative oil permeability
real d_dSo_perm_water (const ConservedVars &CV, const XtraVars &XV) {
  return -2*XV.Sw - PERM_PERT_WATER*(2 * chaos [FLUX] [GLOBAL] [UNIFORM] [1] - 1) * XV.Sw * (2.0 - 3.0*XV.Sw);
}

real lambda_o, lambda_w, d_dSo_lambda_o, d_dSo_lambda_w, VISCOSITY_FRAC;

real calc_max_speed (const ConservedVars &CV, const XtraVars &XV, const int &d)
{
  real buffer = 0;
  
  VISCOSITY_FRAC = OIL_VISCOSITY / WATER_VISCOSITY;
  
  lambda_o = perm_oil   (CV, XV);
  lambda_w = perm_water (CV, XV);
  
  d_dSo_lambda_o = d_dSo_perm_oil   (CV, XV);
  d_dSo_lambda_w = d_dSo_perm_water (CV, XV);
  
  buffer += d_dSo_lambda_o * (lambda_o + VISCOSITY_FRAC * lambda_w);
  buffer -= (d_dSo_lambda_o + VISCOSITY_FRAC * d_dSo_lambda_w) * lambda_o;
  buffer /= sqr(lambda_o + VISCOSITY_FRAC * lambda_w);
  buffer *= TOTAL_FLOW_RATE * ROCK_PERMEABILITY;
  
  return fabs(buffer);
}

void calc_point_flux (ConservedVars &F, const AllVariables &V, const int &d)
{
  VISCOSITY_FRAC = OIL_VISCOSITY / WATER_VISCOSITY;
  F.So = perm_oil (V.U, V.XU) / ( perm_oil (V.U, V.XU) + VISCOSITY_FRAC * perm_water (V.U, V.XU) );
  F.So *= TOTAL_FLOW_RATE * ROCK_PERMEABILITY;
}

void calc_xtra (XtraVars &Xijk, const ConservedVars &Uijk, const int &i, const int &j, const int &k)
{
  Xijk.Sw = 1.0 - Uijk.So;
  // Constraints for XtraVars
  if (Xijk.Sw < 0.0) Xijk.Sw = 0.0;
  if (Xijk.Sw > 1.0) Xijk.Sw = 1.0;
}
