
/* EQUATION SPECIFIC FVM routines for FLUX
 * Karhunen-Loeve expansion of flux (Burgers' equation):
 * f(u) = u^2/2 + Karhunen-Loeve expansion
 */

///::title:: Karhunen-Loeve expansion (Burgers' 1D)
///::vars:: KL=10 KL_DECAY=2.0 KL_SCALING=0.01

#include "flux.h"
#include "equation.h"
#include "chaos.h"

void flux_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = KL;
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL]  = 0;
}

// computes Karhunen-Loeve expansion
real calc_KL_flux (real u) {
  real value = u*u/2.0;
  for (int m=1; m<=KL; m++)
    value += KL_SCALING * pow(m,-KL_DECAY) * sin(m*pi*u) * (2 * chaos [FLUX] [GLOBAL] [UNIFORM] [m-1] - 1);
  return value;
}

void coefficients_init () {}

real calc_max_speed (const ConservedVars &CV, const XtraVars &XV, const int &d)
{
  real value = CV.u;
  for (int m=1; m<=KL; m++)
    value += KL_SCALING * pow(m,-KL_DECAY) * m*pi*cos(m*pi*CV.u) * (2 * chaos [FLUX] [GLOBAL] [UNIFORM] [m-1] - 1);
  
  return fabs(value);
}

void calc_point_flux (ConservedVars &F, const AllVariables &V, const int &d)
{
  F.u = calc_KL_flux(V.U.u);
}

void calc_xtra (XtraVars &Xijk, const ConservedVars &Uijk, const int &i, const int &j, const int &k)
{
  Xijk.E = calc_KL_flux(Uijk.u);
}
