
/* EQUATION SPECIFIC FVM routines for FLUX
 * Uniformly perturbed flux (Burgers' equation):
 * fx(u) = u^px/px, fy(u) = u^py/py, fz(u) = u^pz/pz, px,py,pz ~ U (PMIN, PMAX), independent
 */

///::title:: Uniformly perturbed (Burgers' 1D and 2D)
///::consts:: M_UNIFORM=3 M_NORMAL=0
///::vars:: PMIN=1.0 PMAX=3.0

#include "flux.h"
#include "equation.h"
#include "chaos.h"

void flux_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = 2;
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL]  = 0;
}

void coefficients_init () {}

real calc_max_speed (const ConservedVars &CV, const XtraVars &XV, const int &d)
{
  return pow(fabs(CV.u), PMIN + (PMAX-PMIN) * chaos [FLUX] [GLOBAL] [UNIFORM] [d] - 1);
}

real p;

void calc_point_flux (ConservedVars &F, const AllVariables &V, const int &d)
{
  p = PMIN + (PMAX-PMIN) * chaos [FLUX] [GLOBAL] [UNIFORM] [d];
  F.u = pow(fabs(V.U.u), p) / p;
}

void calc_xtra (XtraVars &Xijk, const ConservedVars &Uijk, const int &i, const int &j, const int &k)
{
  Xijk.E = pow(fabs(Uijk.u), 2) / 2;
}
