
/* EQUATION SPECIFIC FVM routines for FLUX
 * Standard deterministic flux (Euler equations of gas dynamics)
 */

///::title:: Deterministic (Euler equations)
///::consts:: GAMMA=(5.0/3.0)

#include "equation.h"
#include "chaos.h"

#include <iostream>

void flux_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL] = 0;
}

void coefficients_init() {}

real calc_max_speed (const ConservedVars &CV, const XtraVars &XV, const int &d)
{
  return fabs(XV.u[d]) + sqrt( GAMMA * XV.p / CV.rho );
}

void calc_point_flux (ConservedVars &F, const AllVariables &V, const int &d)
{
  F.rho = V.U.m[d];
  F.m = V.U.m * V.XU.u[d];
  F.m[d] += V.XU.p;
  F.E = (V.U.E + V.XU.p) * V.XU.u[d];
}

void calc_xtra (XtraVars &Xijk, const ConservedVars &Uijk, const int &i, const int &j, const int &k)
{
  real ie = Uijk.E - 0.5*Uijk.m.sqr()/Uijk.rho;
  Xijk.u = Uijk.m / Uijk.rho;
  Xijk.p = (GAMMA-1)*ie;
  if (Xijk.p < 0) {
    std::cerr << "Negative Pressure p=" << Xijk.p << "(" << i << ' ' << j << ' ' << k << ") t=" << t << std::endl;
    std::exit(1);
  }
  //Xijk.a = sqrt(GAMMA*Xijk.p/Uijk.rho);
}
