
/* EQUATION SPECIFIC FVM routines for FLUX
 * Uniformly perturbed flux (Euler equations of gas dynamics):
 * gas constant of specific heats becomes GAMMA ~ U (GAMMA_MEAN +/- GAMMA_DEV)
 */

///::title:: Uniformly perturbed specific heats
///::vars:: GAMMA_MEAN=5.0/3.0 GAMMA_DEV=0.1
///::consts:: M_UNIFORM=1

#include "equation.h"
#include "chaos.h"

#include <iostream>

void flux_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = 1;
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL] = 0;
}

real GAMMA;

void coefficients_init () {
    GAMMA = GAMMA_MEAN - GAMMA_DEV + 2 * GAMMA_DEV * chaos [FLUX] [GLOBAL] [UNIFORM] [0];
}

real calc_max_speed (const ConservedVars &CV, const XtraVars &XV, const int &d)
{
  return fabs(XV.u[d]) + sqrt( GAMMA * XV.p / CV.rho );
}

void calc_point_flux (ConservedVars &F, const AllVariables &V, const int &d)
{
  F.rho = V.U.m[d];
  F.m = V.U.m * V.XU.u[d];
  F.m[d] += V.XU.p;
  F.E = (V.U.E + V.XU.p) * V.XU.u[d];
}

void calc_xtra (XtraVars &Xijk, const ConservedVars &Uijk, const int &i, const int &j, const int &k)
{
  if (GAMMA < 1.0) {
    std::cerr << "GAMMA < 1.0" << std::endl;
    std::exit(1);
  }
  real ie = Uijk.E - 0.5*Uijk.m.sqr()/Uijk.rho;
  Xijk.u = Uijk.m / Uijk.rho;
  Xijk.p = (GAMMA-1)*ie;
  if (Xijk.p < 0) {
    std::cerr << "Negative Pressure p=" << Xijk.p << "(" << i << ' ' << j << ' ' << k << ") t=" << t << std::endl;
    std::exit(1);
  }
  //Xijk.a = sqrt(GAMMA*Xijk.p/Uijk.rho);
}
