
/* EQUATION SPECIFIC FVM routines for FLUX
 * Standard deterministic flux (Magneto-HydroDynamics equations)
 */

///::title:: Deterministic (Magneto-HydroDynamics equations)
///::consts:: GAMMA=(5.0/3.0)

#include "equation.h"
#include "chaos.h"

#include <iostream>

void flux_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL] = 0;
}

void coefficients_init () {}

real calc_max_speed (const ConservedVars &CV, const XtraVars &XV, const int &d) {
    return fabs(XV.u[d]) + XV.cf[d];
}

void calc_point_flux (ConservedVars &F, const AllVariables &V, const int &d) {
  
  real tP = V.XU.p + 0.5*V.U.B.sqr();

  F.rho = V.U.m[d];
  F.m = V.U.m * V.XU.u[d] - V.U.B * V.U.B[d];
  F.m[d] += tP;
  F.E = (V.U.E + tP) * V.XU.u[d] - (V.XU.u*V.U.B) * V.U.B[d];
  F.B = V.XU.u[d] * V.U.B - V.XU.u * V.U.B[d];
}

void calc_xtra (XtraVars &Xijk, const ConservedVars &Uijk, const int &i, const int &j, const int &k) {
  
  real ie = Uijk.E - 0.5*Uijk.m.sqr()/Uijk.rho - 0.5*Uijk.B.sqr();
  Xijk.u = Uijk.m / Uijk.rho;
  Xijk.p = (GAMMA-1)*ie;
  if (Xijk.p < 0) {
    std::cerr << "Negative Pressure p=" << Xijk.p << "(" << i << ' ' << j << ' ' << k << ") t=" << t << std::endl;
    std::exit(1);
  }
  Xijk.a = sqrt(GAMMA*Xijk.p/Uijk.rho);
  real cm2 = 0.5*(Xijk.a*Xijk.a + Uijk.B.sqr()/Uijk.rho);
  for (int d=0; d<NDIR; d++) {
    real x = cm2*cm2 - Xijk.a*Xijk.a*Uijk.B[d]*Uijk.B[d]/Uijk.rho;
    Xijk.cf[d] = sqrt(cm2 + ssqrt(x));
  }
}
