
/* EQUATION SPECIFIC FVM routines for FLUX
 * Standard deterministic flux (Shallow Water equations)
 */

///::title:: Deterministic (Shallow Water Equations)

#include "flux.h"

#include "equation.h"
#include "chaos.h"

void flux_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL] = 0;
}

void coefficients_init ()
{
  // initializing bottom topography
  for (int i = 0; i < NXl; i++)
    for (int j = 0; j < NYl; j++)
      Xtra[i][j][0].b = BED_OFFSET + BED_SCALING * b(i,j);
}

real calc_max_speed (const ConservedVars &CV, const XtraVars &XV, const int &d) {
  return fabs(XV.u[d]) + sqrt ( GRAVITY * CV.h );
}

void calc_point_flux(ConservedVars &F, const AllVariables &V, const int &d)
{
  F.h       = V.U.h * V.XU.u[d];
  F.hu[d]   = V.U.h * sqr(V.XU.u[d]) + GRAVITY * sqr(V.U.h) / 2.0;
  F.hu[1-d] = V.U.h * V.XU.u[d] * V.XU.u[1-d];
}

void calc_xtra(XtraVars &Xijk, const ConservedVars &Uijk, const int &i, const int &j, const int &k)
{
  Xijk.u = Uijk.hu / Uijk.h;
  Xijk.E = ( Uijk.h * (Xijk.u * Xijk.u) + GRAVITY * sqr(Uijk.h) ) / 2.0;
  Xijk.b = Xtra[i][j][k].b;
  Xijk.S = Xijk.b + Uijk.h;
}
