
/* EQUATION SPECIFIC FVM routines for FLUX
 * Standard deterministic flux (Wave equation, 2nd type)
 */

///::title:: Deterministic (Wave equation)
///::vars:: ADVECTX=0 ADVECTY=0 ADVECTZ=0

#include "flux.h"

#include "material.h"
#include "equation.h"
#include "chaos.h"

void flux_set_chaos_size () {
  material_set_chaos_size ();
}

void coefficients_init ()
{
  #if BUFFERED_MATERIAL
  material_c_init();
  #endif
  
  for (int i = 0; i < NXl; i++)
    for (int j = 0; j < NYl; j++)
      for (int k = 0; k < NZl; k++)
        Xtra[i][j][k].c = MATERIAL_OFFSET + MATERIAL_SCALING * c(i,j,k);
}

real calc_max_speed (const ConservedVars &CV, const XtraVars &XV, const int &d)
{
  return sqrt(XV.c);
}

void calc_point_flux(ConservedVars &F, const AllVariables &V, const int &d)
{
  #if ISOTROPIC
  F.p    = - V.XU.c * V.U.u[d];
  #else
  F.p    = - V.XU.c[d] * V.U.u;
  #endif
  F.u[d] = - V.U.p;
  
  #if ADVECTX
  if (d == X) F.u -= ADVECTX*V.U.u;
  #endif
  #if ADVECTY
  if (d == Y) F.u -= ADVECTY*V.U.u;
  #endif
  #if ADVECTZ
  if (d == Z) F.u -= ADVECTZ*V.U.u;
  #endif
}

void calc_xtra (XtraVars &Xijk, const ConservedVars &Uijk, const int &i, const int &j, const int &k)
{
  Xijk.c = Xtra[i][j][k].c;
  Xijk.E = sqr( Uijk.p );
}
