
/*
 * Routines for random lognormal material coefficient generation using FFT
 */

#ifndef MATERIAL_FFT_H
#define MATERIAL_FFT_H

#include "numtypes.h"
#include "volumes.h"

// === User-defined routines (defined in model_*.cpp files)

// defines a stationary covariance kernel (for periodic domains, kernel depends not only on distance, but also on direction)
real material_covariance_kernel (const vector &v, const int &layer = 0);

// === Non-user-defined routines (defined in material-fft.cpp or material-fft-mpi.cpp files)

// return volume (buffer) for the buffered material
volume<real> & buffered_material ();

// initialization
void material_fft_init ();

// initializes correlation kernel, computes its FFT transform
void material_fft_level_init ();

// draws one realization of the random lognormal material coefficient
void material_fft_draw_sample ();

// deallocates memory, destructs instances
void material_fft_level_exit();

// finalization
void material_fft_exit ();

#endif
