
#include "material-kl.h"

#include "chaos.h"

#define BASE_FREQ_X 2*pi/MAXX
#define BASE_FREQ_Y 2*pi/MAXY
#define BASE_FREQ_Z 2*pi/MAXZ

real get_kl_uniform (real x, real y, real z, int id, int stream) {
  
  long_int chaos_offset = id * KL;
  
  real value = 0.0;
  
  for (int m=1; m<=KL; m++)
    value += pow(m,-KL_DECAY/2.0) * sin(m*BASE_FREQ_X*x) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(m-1) + chaos_offset] - 1);
  
  return value;
}

real get_kl2d_uniform (real x, real y, real z, int id, int stream) {
  
  long_int chaos_offset = id * KL * (KL + 2);
  
  real value = 0.0;
  
  // mx == 0
  for (int my=1; my<=KL; my++)
    value += pow(my,-KL_DECAY/2.0) * sin(my*BASE_FREQ_Y*y) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(my-1) + chaos_offset] - 1);
  
  chaos_offset += KL;
  
  // my == 0
  for (int mx=1; mx<=KL; mx++)
    value += pow(mx,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_X*x) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(mx-1) + chaos_offset] - 1);
  
  chaos_offset += KL;
  
  // mx, my > 0
  for (int mx=1; mx<=KL; mx++)
    for (int my=1; my<=KL; my++)
      value += pow(mx,-KL_DECAY/2.0)*pow(my,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_X*x)*sin(my*BASE_FREQ_Y*y) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(mx-1)*KL+(my-1) + chaos_offset] - 1);
  
  return value;
}

real get_kl3d_uniform (real x, real y, real z, int id, int stream) {
  
  long_int chaos_offset = id * KL * KL * KL + KL * KL * 3 + KL * 3;
  
  real value = 0.0;
  
  // Karhunen–Loeve expansion of log(c) consisting of KL basis functions
  
  // mx == 0, my == 0
  for (int mz=1; mz<=KL; mz++)
    value += pow(mz,-KL_DECAY/2.0) * sin(mz*BASE_FREQ_Z*z) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(mz-1) + chaos_offset] - 1);
  
  chaos_offset += KL;
  
  // mx == 0, mz == 0
  for (int my=1; my<=KL; my++)
    value += pow(my,-KL_DECAY/2.0) * sin(my*BASE_FREQ_Z*z) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(my-1) + chaos_offset] - 1);
  
  chaos_offset += KL;
  
  // my == 0, mz == 0
  for (int mx=1; mx<=KL; mx++)
    value +=  pow(mx,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_Z*z) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(mx-1) + chaos_offset] - 1);
  
  chaos_offset += KL;
  
  // mx == 0
  for (int my=1; my<=KL; my++)
    for (int mz=1; mz<=KL; mz++)
      value += pow(my,-KL_DECAY/2.0)*pow(mz,-KL_DECAY/2.0) * sin(my*BASE_FREQ_Y*y)*sin(mz*BASE_FREQ_Z*z) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(my-1)*KL + (mz-1) + chaos_offset] - 1);
  
  chaos_offset += KL*KL;
  
  // my == 0
  for (int mx=1; mx<=KL; mx++)
    for (int mz=1; mz<=KL; mz++)
      value += pow(mx,-KL_DECAY/2.0)*pow(mz,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_Y*y)*sin(mz*BASE_FREQ_Z*z) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(mx-1)*KL + (mz-1) + chaos_offset] - 1);
  
  chaos_offset += KL*KL;
  
  // mz == 0
  for (int mx=1; mx<=KL; mx++)
    for (int my=1; my<=KL; my++)
      value += pow(mx,-KL_DECAY/2.0)*pow(my,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_Y*y)*sin(my*BASE_FREQ_Z*z) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(mx-1)*KL + (my-1) + chaos_offset] - 1);
  
  chaos_offset += KL*KL;
  
  // mx, my, mz > 0
  for (int mx=1; mx<=KL; mx++)
    for (int my=1; my<=KL; my++)
      for (int mz=1; mz<=KL; mz++)
        value += pow(mx,-KL_DECAY/2.0)*pow(my,-KL_DECAY/2.0)*pow(mz,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_X*x)*sin(my*BASE_FREQ_Y*y)*sin(mz*BASE_FREQ_Z*z) * (2 * chaos [stream] [GLOBAL] [UNIFORM] [(mx-1)*KL*KL+(my-1)*KL + (mz-1) + chaos_offset] - 1);
  
  return value;
}

real get_kl_normal (real x, real y, real z, int id, int stream) {
  
  long_int chaos_offset = id * KL;
  
  real value = 0.0;
  
  for (int m=1; m<=KL; m++)
    value += pow(m,-KL_DECAY/2.0) * sin(m*BASE_FREQ_X*x) * chaos [stream] [GLOBAL] [NORMAL] [(m-1) + chaos_offset];
  
  return value;
}

real get_kl2d_normal (real x, real y, real z, int id, int stream) {
  
  long_int chaos_offset = id * KL * (KL + 2);
  
  real value = 0.0;
  
  // mx == 0
  for (int my=1; my<=KL; my++)
    value += pow(my,-KL_DECAY/2.0) * sin(my*BASE_FREQ_Y*y) * chaos [stream] [GLOBAL] [NORMAL] [(my-1) + chaos_offset];
  
  chaos_offset += KL;
  
  // my == 0
  for (int mx=1; mx<=KL; mx++)
    value += pow(mx,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_X*x) * chaos [stream] [GLOBAL] [NORMAL] [(mx-1) + chaos_offset];
  
  chaos_offset += KL;
  
  // mx, my > 0
  for (int mx=1; mx<=KL; mx++)
    for (int my=1; my<=KL; my++)
      value += pow(mx,-KL_DECAY/2.0)*pow(my,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_X*x)*sin(my*BASE_FREQ_Y*y) * chaos [stream] [GLOBAL] [NORMAL] [(mx-1)*KL+(my-1) + chaos_offset];
  
  return value;
}

real get_kl3d_normal (real x, real y, real z, int id, int stream) {
  
  long_int chaos_offset = id * KL * KL * KL + KL * KL * 3 + KL * 3;
  
  real value = 0.0;
  
  // Karhunen–Loeve expansion of log(c) consisting of KL basis functions
  
  // mx == 0, my == 0
  for (int mz=1; mz<=KL; mz++)
    value +=  pow(mz,-KL_DECAY/2.0) * sin(mz*BASE_FREQ_Z*z) * chaos [stream] [GLOBAL] [NORMAL] [(mz-1) + chaos_offset];
  
  chaos_offset += KL;
  
  // mx == 0, mz == 0
  for (int my=1; my<=KL; my++)
    value +=  pow(my,-KL_DECAY/2.0) * sin(my*BASE_FREQ_Z*z) * chaos [stream] [GLOBAL] [NORMAL] [(my-1) + chaos_offset];
  
  chaos_offset += KL;
  
  // my == 0, mz == 0
  for (int mx=1; mx<=KL; mx++)
    value +=  pow(mx,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_Z*z) * chaos [stream] [GLOBAL] [NORMAL] [(mx-1) + chaos_offset];
  
  chaos_offset += KL;
  
  // mx == 0
  for (int my=1; my<=KL; my++)
    for (int mz=1; mz<=KL; mz++)
      value +=  pow(my,-KL_DECAY/2.0)*pow(mz,-KL_DECAY/2.0) * sin(my*BASE_FREQ_Y*y)*sin(mz*BASE_FREQ_Z*z) * chaos [stream] [GLOBAL] [NORMAL] [(my-1)*KL + (mz-1) + chaos_offset];
  
  chaos_offset += KL*KL;
  
  // my == 0
  for (int mx=1; mx<=KL; mx++)
    for (int mz=1; mz<=KL; mz++)
      value +=  pow(mx,-KL_DECAY/2.0)*pow(mz,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_Y*y)*sin(mz*BASE_FREQ_Z*z) * chaos [stream] [GLOBAL] [NORMAL] [(mx-1)*KL + (mz-1) + chaos_offset];
  
  chaos_offset += KL*KL;
  
  // mz == 0
  for (int mx=1; mx<=KL; mx++)
    for (int my=1; my<=KL; my++)
      value +=  pow(mx,-KL_DECAY/2.0)*pow(my,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_Y*y)*sin(my*BASE_FREQ_Z*z) * chaos [stream] [GLOBAL] [NORMAL] [(mx-1)*KL + (my-1) + chaos_offset];
  
  chaos_offset += KL*KL;
  
  // mx, my, mz > 0
  for (int mx=1; mx<=KL; mx++)
    for (int my=1; my<=KL; my++)
      for (int mz=1; mz<=KL; mz++)
        value +=  pow(mx,-KL_DECAY/2.0)*pow(my,-KL_DECAY/2.0)*pow(mz,-KL_DECAY/2.0) * sin(mx*BASE_FREQ_X*x)*sin(my*BASE_FREQ_Y*y)*sin(mz*BASE_FREQ_Z*z) * chaos [stream] [GLOBAL] [NORMAL] [(mx-1)*KL*KL+(my-1)*KL + (mz-1) + chaos_offset];
  
  return value;
}
