
/*
 * Routines returning evaluated Karhunen-Loeve expansions in 1d, 2d, and 3d
 */

#ifndef MATERIAL_KL_H
#define MATERIAL_KL_H

#include "numtypes.h"

// returns an evaluated Karhunen-Loeve expansion with uniform random variables in 1d
// different 'id' - different, independent expansions (id = 0, 1, ...)
// 'stream' specifies the RNG stream, e.g. MODEL, FLUX, TOPOGRAPHY, etc.
real get_kl_uniform (real x, real y, real z, int id, int stream);

// returns an evaluated Karhunen-Loeve expansion with uniform random variables in 2d
// different 'id' - different, independent expansions (id = 0, 1, ...)
// 'stream' specifies the RNG stream, e.g. MODEL, FLUX, TOPOGRAPHY, etc.
real get_kl2d_uniform (real x, real y, real z, int id, int stream);

// returns an evaluated Karhunen-Loeve expansion with uniform random variables in 3d
// different 'id' - different, independent expansions (id = 0, 1, ...)
// 'stream' specifies the RNG stream, e.g. MODEL, FLUX, TOPOGRAPHY, etc.
real get_kl3d_uniform (real x, real y, real z, int id, int stream);

// returns an evaluated Karhunen-Loeve expansion with normal random variables in 1d
// different 'id' - different, independent expansions (id = 0, 1, ...)
// 'stream' specifies the RNG stream, e.g. MODEL, FLUX, TOPOGRAPHY, etc.
real get_kl_normal (real x, real y, real z, int id, int stream);

// returns an evaluated Karhunen-Loeve expansion with normal random variables in 2d
// different 'id' - different, independent expansions (id = 0, 1, ...)
// 'stream' specifies the RNG stream, e.g. MODEL, FLUX, TOPOGRAPHY, etc.
real get_kl2d_normal (real x, real y, real z, int id, int stream);

// returns an evaluated Karhunen-Loeve expansion with normal random variables in 3d
// different 'id' - different, independent expansions (id = 0, 1, ...)
// 'stream' specifies the RNG stream, e.g. MODEL, FLUX, TOPOGRAPHY, etc.
real get_kl3d_normal (real x, real y, real z, int id, int stream);

#endif
