
#include "material.h"

#include "equation.h"
#include "quadrature.h"
#include "multi.h"

int MATERIAL_N [NDIR] [L+1];
int MULTI_FFT = 1;

#if ISOTROPIC
real c (const int &i, const int &j, const int &k) {
  
  #if BUFFERED_MATERIAL
  
  return buffered_material() [i][j][k];
  
  #else
  
  // use quadrature rule
  
  real vc = 0.0;
  real qc;
  
  real x = (mod(i + NXl*multix, NXl_FULL) + 0.5) * dx[X];
  real y = (mod(j + NYl*multiy, NYl_FULL) + 0.5) * dx[Y];
  real z = (mod(k + NZl*multiz, NZl_FULL) + 0.5) * dx[Z];
  
  for (int px=0; px<(HAVE_DIR(X) ? QUAD_NUM : 1); px++)
    for (int py=0; py<(HAVE_DIR(Y) ? QUAD_NUM : 1); py++)
      for (int pz=0; pz<(HAVE_DIR(Z) ? QUAD_NUM : 1); pz++) {
        qc = c ( x + QUAD_POINTS[px] * dx[X],
                 y + QUAD_POINTS[py] * dx[Y],
                 z + QUAD_POINTS[pz] * dx[Z] );
        if (HAVE_DIR(X)) qc *= QUAD_WEIGHTS[px];
        if (HAVE_DIR(Y)) qc *= QUAD_WEIGHTS[py];
        if (HAVE_DIR(Z)) qc *= QUAD_WEIGHTS[pz];
        vc += qc;
      }
  
  return vc;
  
  #endif
}

#else
matrix c (const int &i, const int &j, const int &k) {
  
  #if BUFFERED_MATERIAL
  
  // copy from buffer
  return buffered_material() [i][j][k];
  
  #else
  
  // use quadrature rule
  
  matrix vc = 0.0;
  matrix qc;
  real x = (mod(i + NXl*multix, NXl_FULL) + 0.5) * dx[X];
  real y = (mod(j + NYl*multiy, NYl_FULL) + 0.5) * dx[Y];
  real z = (mod(k + NZl*multiz, NZl_FULL) + 0.5) * dx[Z];
  
  for (int px=0; px<(HAVE_DIR(X) ? QUAD_NUM : 1); px++)
    for (int py=0; py<(HAVE_DIR(Y) ? QUAD_NUM : 1); py++)
      for (int pz=0; pz<(HAVE_DIR(Z) ? QUAD_NUM : 1); pz++) {
        qc = c ( x + QUAD_POINTS[px] * dx[X],
                 y + QUAD_POINTS[py] * dx[Y],
                 z + QUAD_POINTS[pz] * dx[Z] );
        if (HAVE_DIR(X)) qc *= QUAD_WEIGHTS[px];
        if (HAVE_DIR(Y)) qc *= QUAD_WEIGHTS[py];
        if (HAVE_DIR(Z)) qc *= QUAD_WEIGHTS[pz];
        vc += qc;
      }
  
  return vc;
  
  #endif
}

#endif
