
#include "material.h"
#include "chaos.h"
#include "mlmc.h"

///::title:: Log-normal PERIODIC DISCONTINUOUS material using FFT
///::vars:: MATERIAL_NX=128 MATERIAL_NY=128 MATERIAL_NZ=128
///::consts:: BUFFERED_MATERIAL=1 SMOOTH_FLUX=0 MATERIAL_LAYERS=4
///::depends:: material-fft
///::macro:: INIT_HOOK material-fft.h material_fft_init();
///::macro:: EXIT_HOOK material-fft.h material_fft_exit();
///::macro:: LEVEL_INIT_HOOK material-fft.h material_fft_level_init();
///::macro:: LEVEL_EXIT_HOOK material-fft.h material_fft_level_exit();
///::vars:: LINKER_LIBS="-lfftw3"

const real MATERIAL_VARIANCE     [MATERIAL_LAYERS] = {0.2, 0.4, 0.2, 0.4};
const int  ISOTROPIC_MATERIAL    [MATERIAL_LAYERS] = {0,     0,   0,   0};
const real  CORRELATION_LENGTH   [MATERIAL_LAYERS] = {0.5, 0.1, 0.5, 0.1};
const real  CORRELATION_LENGTH_X [MATERIAL_LAYERS] = {1.0, 2.0, 1.0, 2.0};
const real  CORRELATION_LENGTH_Y [MATERIAL_LAYERS] = {0.5, 0.1, 0.5, 0.1};
const real  CORRELATION_LENGTH_Z [MATERIAL_LAYERS] = {1.0, 2.0, 1.0, 2.0};

void material_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = MATERIAL_LAYERS * MATERIAL_N[X][SAMPLES_LEVEL()] * MATERIAL_N[Y][SAMPLES_LEVEL()] * MATERIAL_N[Z][SAMPLES_LEVEL()];
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL]  = 0;
}

void material_c_init () {
  material_fft_draw_sample ();
}

real material_covariance_kernel (const vector &v, const int &layer) {
  
  // isotropic case
  if ( ISOTROPIC_MATERIAL [layer] ) {
    vector o = vector (0,0,0);
    return MATERIAL_VARIANCE [layer] * exp ( - sqrt ( sqr(sin(pi/MAXX*dist(v,o,X))) + sqr(sin(pi/MAXY*dist(v,o,Y))) + sqr(sin(pi/MAXZ*dist(v,o,Z))) ) / CORRELATION_LENGTH [layer] );
  }
  
  // anisotropic case
  else {
    vector o = vector (0,0,0);
    return MATERIAL_VARIANCE [layer] * exp ( - sqrt ( sqr(sin(pi/MAXX*dist(v,o,X))) / sqr(CORRELATION_LENGTH_X[layer]) + sqr(sin(pi/MAXY*dist(v,o,Y))) / sqr(CORRELATION_LENGTH_Y[layer]) + sqr(sin(pi/MAXZ*dist(v,o,Z))) / sqr(CORRELATION_LENGTH_Z[layer]) ) );
  }
}


/*
Since BUFFERED_MATERIAL=1, routine c() is NOT needed;
all needed routines are in material-fft.h and material-fft.cpp
*/
