
#include "material.h"
#include "chaos.h"
#include "mlmc.h"

///::title:: Log-normal PERIODIC material coefficient using PARALLEL FFT
///::consts:: NUM_LAYERS=1
///::vars:: MATERIAL_NX=128 MATERIAL_NY=128 MATERIAL_NZ=128
///::vars:: MATERIAL_LAYERS=1 MATERIAL_VARIANCE=1.0 ISOTROPIC_MATERIAL=1 CORRELATION_LENGTH=0.1 CORRELATION_LENGTH_X=2.0 CORRELATION_LENGTH_Y=0.1 CORRELATION_LENGTH_Z=2.0
///::consts:: BUFFERED_MATERIAL=1 SMOOTH_FLUX=0
///::depends:: material-fft-mpi
///::macro:: INIT_HOOK material-fft.h material_fft_init();
///::macro:: EXIT_HOOK material-fft.h material_fft_exit();
///::macro:: LEVEL_INIT_HOOK material-fft.h material_fft_level_init();
///::macro:: LEVEL_EXIT_HOOK material-fft.h material_fft_level_exit();
///::vars:: LINKER_LIBS="-lfftw3_mpi@-lfftw3"

void material_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  #if ENSURE_PARALLEL_REPRODUCIBILITY
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = MATERIAL_N[X][SAMPLES_LEVEL()] * MATERIAL_N[Y][SAMPLES_LEVEL()] * MATERIAL_N[Z][SAMPLES_LEVEL()];
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL]  = 0;
  #else
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL]  = ( MATERIAL_N[X][SAMPLES_LEVEL()] * MATERIAL_N[Y][SAMPLES_LEVEL()] * MATERIAL_N[Z][SAMPLES_LEVEL()] ) / MULTI_FFT;
  #endif
}

void material_c_init () {
  material_fft_draw_sample ();
}

#if ISOTROPIC_MATERIAL

// isotropic case

real material_covariance_kernel (const vector &v, const int &layer) {
  vector o = vector (0,0,0);
  return MATERIAL_VARIANCE * exp ( - sqrt ( sqr(sin(pi/MAXX*dist(v,o,X))) + sqr(sin(pi/MAXY*dist(v,o,Y))) + sqr(sin(pi/MAXZ*dist(v,o,Z))) ) / CORRELATION_LENGTH );
}

#else

// anisotropic case

real material_covariance_kernel (const vector &v, const int &layer) {
  vector o = vector (0,0,0);
  return MATERIAL_VARIANCE * exp ( - sqrt ( sqr(sin(pi/MAXX*dist(v,o,X))) / sqr(CORRELATION_LENGTH_X) + sqr(sin(pi/MAXY*dist(v,o,Y))) / sqr(CORRELATION_LENGTH_Y) + sqr(sin(pi/MAXZ*dist(v,o,Z))) / sqr(CORRELATION_LENGTH_Z) ) );
}

#endif

/*
Since BUFFERED_MATERIAL=1, routine c() is NOT needed;
all needed routines are in material-fft.h and material-fft.cpp
*/
