
#include "material.h"
#include "chaos.h"

#include "material-kl.h"

///::title:: Log-Normal Karhunen-Loeve expansion (2D, discont's)
///::vars:: KL=10 MATERIAL_SCALING=0.1 KL_DECAY=2.5
///::vars:: MEAN_TOP=0.5 MEAN_BOTTOM=2.0 MATERIAL_INTERFACE_POSITION=2.0
///::consts:: SMOOTH_FLUX=0
///::depends:: material-kl

void material_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = 2*KL*(KL+2);
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL]  = 0;
}

real c (real x, real y, real z) {
  
  // Karhunen–Loeve expansion of log(c) consisting of KL basis functions
  
  // bottom subdomain
  if (y < MATERIAL_INTERFACE_POSITION)
    return MEAN_BOTTOM + exp ( get_kl2d_normal (x, y, z, 0, FLUX) );
  
  // top subdomain
  else
    return MEAN_TOP + exp ( get_kl2d_normal (x, y, z, 1, FLUX) );
}
