
#include "material.h"
#include "chaos.h"

#include "material-kl.h"

///::title:: Log-Normal Karhunen-Loeve expansion (3D, discont's)
///::vars:: KL=10 MATERIAL_SCALING=0.1 KL_DECAY=2.5
///::vars:: MEAN_BOTTOM=2.0 MEAN_LEFT=0.5 MEAN_RIGHT=0.1 MATERIAL_INTERFACE_POSITION_Z=2.0 MATERIAL_INTERFACE_POSITION_Y=1.5
///::consts:: SMOOTH_FLUX=0
///::depends:: material-kl

void material_set_chaos_size () {
  CHAOS_SIZE [FLUX] [GLOBAL] [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [GLOBAL] [NORMAL]  = 3*(KL*KL*KL+KL*KL*3+KL*3);
  CHAOS_SIZE [FLUX] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [FLUX] [LOCAL]  [NORMAL]  = 0;
}

real c (real x, real y, real z) {
  
  // Karhunen–Loeve expansion of log(c) consisting of KL basis functions
  
  // bottom subdomain
  if (z < MATERIAL_INTERFACE_POSITION_Z)
    return MEAN_BOTTOM + exp ( get_kl3d_normal (x, y, z, 0, FLUX) );
  
  else {
    
    // left subdomain
    if (y < MATERIAL_INTERFACE_POSITION_Y)
      return MEAN_LEFT + exp ( get_kl3d_normal (x, y, z, 1, FLUX) );
    
    // right subdomain
    else
      return MEAN_RIGHT + exp ( get_kl3d_normal (x, y, z, 2, FLUX) );
  }
}
