
#include "equation.h"
#include "chaos.h"

///::title:: Cloud Shock (2D, modified with all Y_i = 0.5)
///::vars:: NX=128 NY=128 MAXT=0.06
///::consts:: MAXX=1 MAXY=1
///::bc:: NEUMANN

void model_set_chaos_size () {
  CHAOS_SIZE [MODEL] [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [NORMAL]  = 0;
}

void initial_data (PrimitiveVars &v, real x, real y, real z)
{
  if (x < 0.04 + 0.02 * 0.5) {
    v.rho = 3.86859 + 0.1 * 0.5;
    v.u[X] = 11.2536;
    v.p = 167.345 + 1 * 0.5;
  } else {
    real r = sqr(x-0.25) + sqr(y-0.5);
    real phi = (x-0.25) / sqrt(r);
    real r_max = sqr( 0.13 + 0.02 * 0.5 * sin(phi) + 0.01 * 0.5 * sin(10*phi) );
    if ( r <= r_max ) {
      v.rho = 10.0;
      v.rho += 0.5 * 0.5;
      v.rho += 1.0 * 0.5 * sin(4*(x-0.25));
      v.rho += 0.5 * 0.5 * cos(8*(y-0.5));
    }
    else
      v.rho = 1.0;
    v.p = 1.0;
  }
}
