
#include "equation.h"
#include "chaos.h"

///::title:: Cloud Shock (2D, perturbed deterministic)
///::vars:: NX=128 NY=128 MAXT=0.06 PERTURBATION=1e-3
///::consts:: MAXX=1 MAXY=1
///::bc:: NEUMANN

void model_set_chaos_size () {
  CHAOS_SIZE [MODEL] [GLOBAL] [UNIFORM] = 8;
  CHAOS_SIZE [MODEL] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [NORMAL]  = 0;
}

void initial_data (PrimitiveVars &v, real x, real y, real z)
{
  if (x < 0.05 + PERTURBATION * chaos [MODEL] [GLOBAL] [UNIFORM] [0]) {
    v.rho = 3.86859 + PERTURBATION * chaos [MODEL] [GLOBAL] [UNIFORM] [6];
    v.u[X] = 11.2536;
    v.p = 167.345 + PERTURBATION * chaos [MODEL] [GLOBAL] [UNIFORM] [7];
  } else {
    real r = sqr(x-0.25) + sqr(y-0.5);
    real phi = (x-0.25) / sqrt(r);
    real r_max = sqr( 0.13 + PERTURBATION * chaos [MODEL] [GLOBAL] [UNIFORM] [4] * sin(phi) + PERTURBATION * chaos [MODEL] [GLOBAL] [UNIFORM] [5] * sin(10*phi) );
    if ( r <= r_max ) {
      v.rho = 10.0;
      v.rho += PERTURBATION * chaos [MODEL] [GLOBAL] [UNIFORM] [1];
      v.rho += PERTURBATION * chaos [MODEL] [GLOBAL] [UNIFORM] [2] * sin(4*(x-0.25));
      v.rho += PERTURBATION * chaos [MODEL] [GLOBAL] [UNIFORM] [3] * cos(8*(y-0.5));
    }
    else
      v.rho = 1.0;
    v.p = 1.0;
  }
}
