
#include "equation.h"
#include "chaos.h"

///::title:: Kelvin-Helmholz instability (2D, perturbed deterministic)
///::vars:: NX=128 NY=128 MAXT=5.0 PERTURBATION=1e-2
///::consts:: MAXX=1 MAXY=1
///::bc:: REPEAT

void model_set_chaos_size () {
  CHAOS_SIZE [MODEL] [GLOBAL] [UNIFORM] = 4;
  CHAOS_SIZE [MODEL] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [NORMAL]  = 0;
}

void initial_data (PrimitiveVars &v, real x, real y, real z)
{
  // instability
  if (y < 0.25 + PERTURBATION * ( 2 * chaos [MODEL] [GLOBAL] [UNIFORM] [0] - 1 ) * sin(2*pi*x) || y > 0.75 + PERTURBATION * ( 2 * chaos [MODEL] [GLOBAL] [UNIFORM] [0] - 1 ) * sin(2*pi*x)) {
    v.rho = 1;
    v.u[X] = 0.5;
    v.u[Y] = PERTURBATION * ( 2 * chaos [MODEL] [GLOBAL] [UNIFORM] [2] - 1 ) * sin(2*pi*x);
    v.p = 2.5;
  } else {
    v.rho = 2;
    v.u[X] = -0.5;
    v.u[Y] = PERTURBATION * ( 2 * chaos [MODEL] [GLOBAL] [UNIFORM] [3] - 1 ) * sin(2*pi*x);
    v.p = 2.5;
  }
  /*
  // additional perturbation
  if ( fabs(x-0.25) < dx[X] && fabs(y-0.25) < dx[Y] ) v.u[Y] += PERTURBATION * ( 2 * chaos.uniform[0] - 1 );
  if ( fabs(x-0.25) < dx[X] && fabs(y-0.75) < dx[Y] ) v.u[Y] += PERTURBATION * ( 2 * chaos.uniform[1] - 1 );
  if ( fabs(x-0.75) < dx[X] && fabs(y-0.25) < dx[Y] ) v.u[Y] += PERTURBATION * ( 2 * chaos.uniform[2] - 1 );
  if ( fabs(x-0.75) < dx[X] && fabs(y-0.75) < dx[Y] ) v.u[Y] += PERTURBATION * ( 2 * chaos.uniform[3] - 1 );
  */
}
