
#include "equation.h"
#include "chaos.h"
#include <cstdlib>

///::title:: Shock vortex interaction (2D)
///::vars:: NX=128 NY=128 MAXT=0.35
///::consts:: MAXX=1.0 MAXY=1.0
///::bc:: NEUMANN

void model_set_chaos_size () {
  CHAOS_SIZE [MODEL] [GLOBAL] [UNIFORM] = 1;
  CHAOS_SIZE [MODEL] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [NORMAL]  = 0;
}

void initial_data (PrimitiveVars &v, real x, real y, real z)
{ 
  real epsilon=0.3, r_c=0.05, alpha=0.204, x_c=0.25, y_c=0.5, M=1.1;
  
  // shock part
  if (x < 0.5 + 0.1 * chaos [MODEL] [GLOBAL] [UNIFORM] [0]) {
    v.rho  = 1.0;
    v.u[X] = sqrt(GAMMA);
    v.u[Y] = 0.0;
    v.p    = 1.0;
  } else {
    v.rho  = 1.0 / 1.1;
    v.u[X] = 1.1 * sqrt(GAMMA);
    v.u[Y] = 0.0;
    v.p    = 1 - 0.1 * GAMMA;
  }
  
  // vortex part
  if (x < 0.5 + 0.1 * chaos [MODEL] [GLOBAL] [UNIFORM] [0]) {
    real tau       = sqrt( sqr(x - x_c) + sqr(y - y_c) ) / r_c;
    real sin_theta = (y - y_c) / (tau * r_c);
    real cos_theta = (x - x_c) / (tau * r_c);
    
    v.u[X] +=  epsilon * tau * exp( alpha*(1-sqr(tau)) ) * sin_theta;
    v.u[Y] += -epsilon * tau * exp( alpha*(1-sqr(tau)) ) * cos_theta;
    v.p    += -(GAMMA-1) * sqr(epsilon) * exp( 2*alpha*(1-sqr(tau)) ) / ( 4 * alpha * GAMMA ) * v.rho;
    if (v.p < 0) {
      std::cout << " :: ERROR: Negative INITIAL pressure!" << std::endl;
      std::exit(1);
    }
  }
}
