
#include "equation.h"
#include "chaos.h"

///::title:: Cloud Shock (3D)
///::vars:: NX=64 NY=64 NZ=64 MAXT=0.06
///::consts:: MAXX=1 MAXY=1 MAXZ=1
///::bc:: NEUMANN

void model_set_chaos_size () {
  CHAOS_SIZE [MODEL] [GLOBAL] [UNIFORM] = 15;
  CHAOS_SIZE [MODEL] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [NORMAL]  = 0;
}

void initial_data (PrimitiveVars &v, real x, real y, real z)
{
  if (x < 0.04 + 0.02 * chaos [MODEL] [GLOBAL] [UNIFORM] [0]) {
    v.rho = 3.86859 + 0.1 * chaos [MODEL] [GLOBAL] [UNIFORM] [1];
    v.u[X] = 11.2536 + 0.5 * chaos [MODEL] [GLOBAL] [UNIFORM] [12];
    v.B[Y] = 2.1826182 + 0.1 * chaos [MODEL] [GLOBAL] [UNIFORM] [13];
    v.B[Z] = -2.1826182 + 0.1 * chaos [MODEL] [GLOBAL] [UNIFORM] [14];
    v.p = 167.345 + 5 * chaos [MODEL] [GLOBAL] [UNIFORM] [2];
  } else {
    real r = sqr(x-0.25) + sqr(y-0.5) + sqr(z-0.5);
    real phi = (x-0.25) / sqrt(r);
    real xi  = (y-0.5)  / sqrt(r);
    real r_max = sqr( 0.13 + 0.02 * chaos [MODEL] [GLOBAL] [UNIFORM] [3] * sin(phi) + 0.01 * chaos [MODEL] [GLOBAL] [UNIFORM] [4] * sin(10*phi) + 0.01 * chaos [MODEL] [GLOBAL] [UNIFORM] [5] * sin(10*xi) );
    if ( r <= r_max ) {
      v.rho = 10.0;
      v.rho += 0.5 * chaos [MODEL] [GLOBAL] [UNIFORM] [6];
      v.rho += 1.0 * chaos [MODEL] [GLOBAL] [UNIFORM] [7] * sin(4*(x-0.25));
      v.rho += 0.5 * chaos [MODEL] [GLOBAL] [UNIFORM] [8] * cos(8*(y-0.5));
      v.rho += 1.0 * chaos [MODEL] [GLOBAL] [UNIFORM] [9] * sin(6*(z-0.5));
    }
    else
      v.rho = 1.0;
    v.B[Y] = 0.56418958 + 0.05 * chaos [MODEL] [GLOBAL] [UNIFORM] [10];
    v.B[Z] = 0.56418958 + 0.05 * chaos [MODEL] [GLOBAL] [UNIFORM] [11];
    v.p = 1.0;
  }
}
