
#include "equation.h"
#include "chaos.h"

///::title:: Godunov-Powell Magnetic Advection (1D)
///::vars:: NX=512 MAXT=1.0
///::consts:: MAXX=2.0
///::bc:: REPEAT

void model_set_chaos_size () {
  CHAOS_SIZE [MODEL] [GLOBAL] [UNIFORM] = 1;
  CHAOS_SIZE [MODEL] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [NORMAL]  = 0;
}

void initial_data (PrimitiveVars &v, real x, real y, real z)
{
  x *= pi;
  
  real epsilon = 0.5 * chaos [MODEL] [GLOBAL] [UNIFORM] [0];
  real freq    = 50;
  
  v.rho  = 1.0;
  v.u[X] = 1.0;
  v.u[Y] = 0.0;
  v.u[Z] = 0.0;
  v.B[X] = 1.0 - 0.5*sin(x) + epsilon*sin(freq*x);
  v.B[Y] = 0.5;
  v.B[Z] = 0.0;
  v.p    = 0.5;
}
