
#include "equation.h"
#include "chaos.h"

///::title:: Rotor Problem (2D)
///::vars:: NX=128 NY=128 MAXT=0.295
///::consts:: MAXX=1 MAXY=1
///::bc:: NEUMANN

void model_set_chaos_size () {
  CHAOS_SIZE [MODEL] [GLOBAL] [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [NORMAL]  = 0;
}

static inline real f(real r) {
  return (23-200*r)/3;
}

void initial_data (PrimitiveVars &v, real x, real y, real z)
{
  real r = sqrt((x-0.5)*(x-0.5) + (y-0.5)*(y-0.5));

  if (r < 0.1) {
    v.rho = 10;
    v.u[X] = -(10*y-5);
    v.u[Y] = (10*x-5);
  } else if (r < 0.115) {
    v.rho = 1 + 9 * f(r);
    v.u[X] = -(10*y-5) * f(r);
    v.u[Y] = (10*x-5) * f(r);
  } else {
    v.rho = 1;
  }
  v.p = 0.5;
  v.B[X] = 2.5/sqrt(4*pi);
}
