
#include "equation.h"
#include "chaos.h"

///::title:: Sonar wave propagating downwards (2D, zero initial velocity)
///::vars:: NX=128 NY=128 MAXT=0.4
///::consts:: MAXX=2.0 MAXY=1.0
///::vars:: SMOOTH_SOLUTION=1
///::bc:: WAVE_REFLECT

const real AMPLITUDE = 10.0;

const real CENTER_1 = 0.5;
const real CENTER_2 = 1.5;
const real SIGMA  = 0.01;

void model_set_chaos_size () {
  CHAOS_SIZE [MODEL] [GLOBAL] [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [NORMAL]  = 0;
}

void initial_data (PrimitiveVars &v, real x, real y, real z)
{
  v.p += exp ( - ( sqr(x - CENTER_1) + sqr(y - MAXY) ) / (2 * sqr(SIGMA)) );
  v.p += exp ( - ( sqr(x - CENTER_2) + sqr(y - MAXY) ) / (2 * sqr(SIGMA)) );
  v.p *= AMPLITUDE;
}
