
#ifndef MPI_HELPER
#define MPI_HELPER

#include "config.h"
#include "numtypes.h"

// default parallelization parameters
extern int DEFAULT_MULTI [NDIR+1] [L+1];
extern int DEFAULT_MULTIN [L+1];

// MPI ranks (in comm_main) of root cores in comm_level communicators
extern int comm_level_root_ranks [L+1];

// MPI ranks (in comm_main) of last cores in comm_level communicators
extern int comm_level_last_ranks [L+1];

// indicates for which multi_level and mesh_level the parallelization setup was modified
extern int MULTI_MODIFIED [L+1][L+1];

// compute parallelization parameters DEFAULT_MULTI[*][*] and LOWEST_MULTI_LEVEL using MULTI*
void multi_initialize_default_parallelization_setup ();

// compute parallelization parameters DEFAULT_MULTI[*][*] and LOWEST_MULTI_LEVEL using MULTI_CORES
void multi_initialize_default_parallelization_setup_from_MULTI_CORES ();

// report parallelization setup
void multi_report_parallelization_setup ();

// check consistency in number of cores
void multi_check_core_consistency ();

// compute actual parallelization parameters MULTI[*][*][*]
void multi_compute_parallelization_setup ();

// check if modifications in parallelization setup are needed
bool multi_modification_needed (const int multi_level, const int MESH_LEVEL);

// check if modifications in parallelization setup are possible
bool multi_modification_possible (const int multi_level, const int MESH_LEVEL);

// reduce MULTI [X-Z] [multi_level] [MESH_LEVEL] and increase MULTI [M] [multi_level] [MESH_LEVEL]
void multi_modify_ddm (const int multi_level, const int MESH_LEVEL);

// compute comm_level_root_ranks[*]
void compute_comm_level_root_ranks ();

// compute comm_level_last_ranks[*]
void compute_comm_level_last_ranks ();

// set MULTI_LEVEL and multi_level_root
// must be after compute_comm_level_root_ranks()
void set_multi_level ();

// computes the rank of the core (in comm_main)
// with the required 'multix', 'multiy', 'multiz' in the DDM of specified multi_sampler
// in MULTI_LEVEL = L at parallelization configuration MESH_LEVEL = L
int compute_ddm_rank_send (const int &multix, const int &multiy, const int &multiz, const int &multi_sampler = 0);

// computes the rank of the core (in comm_main)
// with the required 'multix', 'multiy', 'multiz' in the DDM of the specified multi_sampler
// for specified 'multi_level' and the current parallelization configuration MESH_LEVEL
int compute_ddm_rank_recv (const int &multix, const int &multiy, const int &multiz, const int &multi_level, const int &multi_sampler = 0);

#endif
