
#include "multi-mpi-timers.h"
#include "multi-mpi-helper.h"

#include "multi_mpi.h"
#include "multi.h"
#include "mlmc.h"
#include "balancer.h"

#include <iostream>
#include <fstream>
#include <cstdlib>
#include "mpi.h"

Timer timer_mpi_ddm, timer_mpi_mc, timer_mpi_mlmc, timer_mpi_idle, timer_mpi_bcast_loads;

double wallclock_runtime_imbalance, wallclock_runtime_mpi_ddm_loads;

void multi_compute_imbalance () {
  wallclock_runtime_imbalance = timer.duration();
}

void multi_compute_idling () {
  
  #ifndef QUIET
  if (multi_main) std::cout << " :: Synchronizing...";
  if (multi_main) std::cout.flush();
  #endif
  
  timer_mpi_idle.start();
  
  multi_sync();
  
  timer_mpi_idle.stop();
  
  #ifndef QUIET
  if (multi_main) std::cout << " done." << std::endl << std::endl;
  #endif
}

void multi_compute_mpi_ddm_loads () {
  wallclock_runtime_mpi_ddm_loads = timer_mpi_ddm.duration() / (MULTIN[L][L] * MULTI[M][L][L]) * MULTI_TOTAL;
  timer_mpi_ddm.reset();
}

void multi_report_runtimes () {
  
  // computing global routine-specific MPI timers
  
  double wallclock_runtime_global_loads;
  double wallclock_runtime_global_mpi_ddm_loads;
  double wallclock_runtime_global_mpi_bcast_loads;
  double wallclock_runtime_global_balancer;
  
  MPI_Reduce (timer_balancer_estimate.duration_ptr(),   &wallclock_runtime_global_loads,           1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
  MPI_Reduce (&wallclock_runtime_mpi_ddm_loads,         &wallclock_runtime_global_mpi_ddm_loads,   1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
  MPI_Reduce (timer_mpi_bcast_loads.duration_ptr(),     &wallclock_runtime_global_mpi_bcast_loads, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
  MPI_Reduce (timer_balancer_distribute.duration_ptr(), &wallclock_runtime_global_balancer,        1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
  
  double wallclock_runtime_global_mpi_ddm;
  double wallclock_runtime_global_mpi_mc;
  double wallclock_runtime_global_mpi_mlmc;
  double wallclock_runtime_global_mpi_idle;
  
  MPI_Reduce (timer_mpi_ddm.duration_ptr(),             &wallclock_runtime_global_mpi_ddm,         1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
  MPI_Reduce (timer_mpi_mc.duration_ptr(),              &wallclock_runtime_global_mpi_mc,          1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
  MPI_Reduce (timer_mpi_mlmc.duration_ptr(),            &wallclock_runtime_global_mpi_mlmc,        1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
  MPI_Reduce (timer_mpi_idle.duration_ptr(),            &wallclock_runtime_global_mpi_idle,        1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
  
  // gather wallclock_runtime_imbalance from each core
  std::vector<double> wallclock_runtime_imbalance_gathered;
  wallclock_runtime_imbalance_gathered .resize (MULTI_TOTAL);
  MPI_Gather (&wallclock_runtime_imbalance, 1, MPI_DOUBLE, wallclock_runtime_imbalance_gathered .data(), 1, MPI_DOUBLE, 0, MPI_COMM_WORLD);
  
  if (multi_main) {
    
    std::cout << std::endl;
    
    std::cout << " :: Wall-clock runtime          : ";
    print_runtime ( timer.duration() );
    std::cout << std::endl;
    
    double wallclock_runtime_global = timer.duration() * MULTI_TOTAL;
    
    std::cout << " :: Wall-clock runtime (global) : ";
    print_runtime ( wallclock_runtime_global );
    std::cout << std::endl;
    
    std::cout << "  : -> Total balancer runtime   : ";
    std::cout << round(1000 * (wallclock_runtime_global_loads + wallclock_runtime_global_balancer) / wallclock_runtime_global)/10 << "%";
    std::cout << std::endl;
    
    #if ESTIMATE_LOADS
    
    std::cout << "  : -> Computation of loads     : ";
    std::cout << round(1000 * wallclock_runtime_global_loads / wallclock_runtime_global)/10 << "%";
    std::cout << std::endl;
    
    std::cout << "    : -> DDM   : ";
    std::cout << round(1000 * wallclock_runtime_global_mpi_ddm_loads / wallclock_runtime_global)/10 << "%";
    std::cout << std::endl;
    
    std::cout << "    : -> Bcast : ";
    std::cout << round(1000 * wallclock_runtime_global_mpi_bcast_loads / wallclock_runtime_global)/10 << "%";
    std::cout << std::endl;
    
    std::cout << "  : -> Distribution of loads    : ";
    std::cout << round(1000 * wallclock_runtime_global_balancer / wallclock_runtime_global)/10 << "%";
    std::cout << std::endl;
    
    #endif
    
    #if USE_ESTIMATED_LOADS
    std::cout << "  :   Computing loads:    " << round(1000 * wallclock_runtime_global_loads    / wallclock_runtime_global)/10 << "%" << std::endl;
    std::cout << "  :      -> DDM:   " << round(1000 * wallclock_runtime_global_mpi_ddm_loads   / wallclock_runtime_global)/10 << "%" << std::endl;
    std::cout << "  :      -> Bcast: " << round(1000 * wallclock_runtime_global_mpi_bcast_loads / wallclock_runtime_global)/10 << "%" << std::endl;
    std::cout << "  :   Distributing loads: " << round(1000 * wallclock_runtime_global_balancer / wallclock_runtime_global)/10 << "%" << std::endl;
    #endif
    
    double wallclock_runtime_global_mpi = 0;
    // += ALL other balancer routines
    wallclock_runtime_global_mpi += wallclock_runtime_global_mpi_ddm;
    wallclock_runtime_global_mpi += wallclock_runtime_global_mpi_mc;
    wallclock_runtime_global_mpi += wallclock_runtime_global_mpi_mlmc;
    wallclock_runtime_global_mpi += wallclock_runtime_global_mpi_idle;
    
    std::cout << std::endl << " :: MPI overhead                : ";
    print_runtime (wallclock_runtime_global_mpi);
    std::cout << std::endl;
    
    std::cout << "  :   DDM:   " << round(1000 * wallclock_runtime_global_mpi_ddm  / wallclock_runtime_global)/10 << "%" << std::endl;
    std::cout << "  :   MC:    " << round(1000 * wallclock_runtime_global_mpi_mc   / wallclock_runtime_global)/10 << "%" << std::endl;
    std::cout << "  :   MLMC:  " << round(1000 * wallclock_runtime_global_mpi_mlmc / wallclock_runtime_global)/10 << "%" << std::endl;
    std::cout << "  :   Idle:  " << round(1000 * wallclock_runtime_global_mpi_idle / wallclock_runtime_global)/10 << "%" << std::endl;
    
    //double mpi_efficiency = 1 - (wallclock_runtime_global_mpi + wallclock_runtime_global_balancer) / wallclock_runtime_global;
    double mpi_efficiency = 1 - wallclock_runtime_global_mpi / wallclock_runtime_global;
    if (mpi_efficiency < 0) mpi_efficiency = 0;
    
    std::cout << " :: MPI efficiency              : " << round(1000*mpi_efficiency)/10 << "%" << std::endl;
    std::cout << std::endl;
    
    std::ofstream runtimef (RUNTIME_FILE);
    
    runtimef << "wallclock_runtime="                        << timer.duration()                         << std::endl;
    runtimef << "wallclock_runtime_global="                 << wallclock_runtime_global                 << std::endl;
    runtimef << "wallclock_runtime_global_loads="           << wallclock_runtime_global_loads           << std::endl;
    runtimef << "wallclock_runtime_global_mpi_ddm_loads="   << wallclock_runtime_global_mpi_ddm_loads   << std::endl;
    runtimef << "wallclock_runtime_global_mpi_bcast_loads=" << wallclock_runtime_global_mpi_bcast_loads << std::endl;
    runtimef << "wallclock_runtime_global_balancer="        << wallclock_runtime_global_balancer        << std::endl;
    runtimef << "wallclock_runtime_global_mpi_ddm="         << wallclock_runtime_global_mpi_ddm         << std::endl;
    runtimef << "wallclock_runtime_global_mpi_mc="          << wallclock_runtime_global_mpi_mc          << std::endl;
    runtimef << "wallclock_runtime_global_mpi_mlmc="        << wallclock_runtime_global_mpi_mlmc        << std::endl;
    runtimef << "wallclock_runtime_global_mpi_idle="        << wallclock_runtime_global_mpi_idle        << std::endl;
    runtimef << "wallclock_runtime_global_mpi="             << wallclock_runtime_global_mpi             << std::endl;
    runtimef << "mpi_efficiency="                           << mpi_efficiency                           << std::endl;
    runtimef << "cores="                                    << MULTI_TOTAL                              << std::endl;
    
    runtimef << "wallclock_runtime_gathered_imbalance=[";
    for (int i = 0; i < wallclock_runtime_imbalance_gathered.size(); i++)
      runtimef << wallclock_runtime_imbalance_gathered[i] << ",";
    runtimef << "]" << std::endl;
    
    runtimef << "default_parallelization=[";
    for (int multi_level = L; multi_level >= LOWEST_MULTI_LEVEL; multi_level--) {
      runtimef << "[";
      for (int par = 0; par < NDIR+1; par++)
        runtimef << DEFAULT_MULTI[par][multi_level] << ",";
      runtimef << "],";
    }
    runtimef << "]" << std::endl;
    
    runtimef.flush();
  }
}
