
#ifndef MULTI_H
#define MULTI_H

#include "numtypes.h"
#include "timer.h"
#include "structures.h"

extern int MULTI[NDIR+1][L+1][L+1];     // MULTI[X-Z][multi_level][mesh_level] - number of cores working on the same domain (in each direction 'X-Z') at multi level 'multi_level' for mesh level 'mesh_level'
static const int M = 3;                 // MULTI[M][multi_level] - number of samplers working at multi level 'multi_level'
extern int MULTIN[L+1][L+1];            // MULTIN[multi_level][mesh_level] - number of cores working on the same domain at multi level 'multi_level' for mesh level 'mesh_level'

extern int MULTI_LEVEL;                 // multi_level of current core
extern int MULTI_SAMPLER;               // multi_sampler of current core
extern int MULTI_SUBDOMAIN;             // multi_subdomain of current core
extern int LOWEST_MULTI_LEVEL;          // lowest multi-core level; all lower levels are computed on the same core as this one

extern int MULTIXl, MULTIYl, MULTIZl;   // number of subdomains in a current level (in each direction)
extern int MULTINl;                     // number of subdomains in a current level (total)
extern int MULTIMl;                     // number core groups working on the same level samples
extern int MULTI_TOTAL;                 // total number of cores that are needed for specified parallelization MLMC

extern int multix, multiy, multiz;      // sub-domain index with communicator's topology
extern int multi_main;                  // boolean to indicate if current core is the root
extern int multi_domain_root;           // boolean to indicate if current core is the root of 'domain' communicator
extern int multi_samplers_root;         // boolean to indicate if current core is the root of 'samplers' communicator
extern int multi_level_root;            // boolean to indicate if current core is the root of 'level' communicator

const  int HEAD = 0, TAIL = 1;          // indicators for the 'HEAD' and 'TAIL' communication between cores in DDM
extern int multi_bc[NDIR][2];           // indicators for communication boundary in DDM

extern Timer timer;                     // timer

// === general parallelization routines -- implemented in multi.cpp file

// outputs the formatted runtime to STDOUT
void print_runtime (real runtime);

// === parallelization-specific routines -- implemented in corresponding multi_*.cpp file

// initializes paralellization back-end, sets 'multi_main'
void multi_init (int argc, char **argv);

// sync all cores
void multi_sync ();

// sync all cores withing the same domain
void multi_sync_domain ();

// check compatibility
void multi_compatibility_checks ();

// computes various mesh-size independent parameters for parallelization of MLMC estimator
// sets MULTI[*][*][*], MULTI_TOTAL, MULTI*l, MULTINl, LOWEST_MULTI_LEVEL, etc.
// must be after 'multi_compatibility_checks()'
void multi_mlmc_init ();

// initializes parallelization setup (MULTI*l) for current MESH_LEVEL
// initializes ONLY parameters (does NOT create any communicators)
// must be after 'multi_mlmc_init()'
void multi_level_init_parameters (const int mesh_level);

// initializes parallelization setup (MULTI*l) for current MESH_LEVEL
// initializes parameters AND creates communicators
// must be after 'multi_mlmc_init()'
void multi_level_init ();

// finalizes parallelization setup
void multi_level_exit ();

// sets up the mesh-size dependent exchange of multi-boundaries
// must be after 'mesh_level_init()'
void multi_mesh_level_init ();

// finalizes the mesh-size dependent exchange of multi-boundaries
void multi_mesh_level_exit ();

// start parallel communication of multi boundaries for XtraVars
void multi_ddm_start_send_recv_xtra ();

// returns minimal timestep throughout the entire domain
real multi_dt (real dt);

// start parallel communication of multi boundaries for ConvervedVars
void multi_ddm_start_send_recv ();

// confirm receives of multi boundaries for ConvervedVars
void multi_ddm_confirm_recv ();

// confirm sends of multi boundaries for ConvervedVars
void multi_ddm_confirm_send ();

// returns sum of functional values from each sub-domain
real multi_reduce_functional (const real value);

// compute load imbalance
void multi_compute_imbalance ();

// sync and compute idling
void multi_compute_idling ();

// compute the parallelization overhead for load computation
void multi_compute_mpi_ddm_loads ();

// combines parts of MC estimates into full MC estimate
void multi_combine_mc ();

// combines MC estimates into MLMC estimate
void multi_combine_mlmc ();

// gather loads from all cores (required in balancer)
void multi_balancer_gather_loads ();

// clean up routines
void multi_cleanup ();

// finalizing calls
void multi_finalize ();

// timing back-end for the Timer class in "timer.h"
double multi_time ();

// report runtimes
void multi_report_runtimes ();

#endif
