
#ifndef MULTI_MPI_H
#define MULTI_MPI_H

#define mpi_REAL MPI_DOUBLE
#define mpi_LONG_INT MPI_UNSIGNED_LONG

#include "numtypes.h"
#include "structures.h"

#include "mpi.h"

// MPI groups and communicators
extern MPI_Group group_domain;
extern MPI_Comm  comm_domain;

// MPI ranks of current core within various MPI communicators
extern int rank_main, rank_domain;

// ranks of cores for communication boundary in DDM
extern int multi_bc_rank[NDIR][2];

// create level communicators
void multi_create_level_communicators ();

// create domain communicators
// must be executed after 'multi_create_entire_level_communicators()' and 'multi_level_init()'
void multi_create_domain_communicators ();

// create samplers communicators
// must be executed after 'multi_create_entire_level_communicators()' and 'multi_level_init()'
void multi_create_samplers_communicators ();

// free level communicators
void multi_free_level_communicators ();

// free domain communicators
void multi_free_domain_communicators ();

// free samplers communicators
void multi_free_samplers_communicators ();

// initialize multi_bc[NDIR][2], multi_bc_rank[NDIR][2]
// must be executed after 'multi_level_init()'
void multi_init_bc_parameters ();

// compute size of communication buffer for ConservedVars
// for the specified multi_level and parallelization setup for the specified mesh_level
int COUNT_CV (const int &multi_level, const int &mesh_level);

// compute size of communication buffer for XtraVars
int COUNT_XV (const int &multi_level, const int &mesh_level);

// compute size of communication buffer for functional
int COUNT_FUNCTIONAL ();

// "SEND" part of 'multi_combine_mc()'
void multi_combine_mc_send ();

// "RECV" part of 'multi_combine_mc()'
void multi_combine_mc_recv ();

// "REDUCE" part of 'multi_combine_mc()'
void multi_combine_mc_reduce();

// "SEND" part of 'multi_combine_mlmc()'
void multi_combine_mlmc_send ();

// "RECV" part of 'multi_combine_mlmc()'
void multi_combine_mlmc_recv ();

#endif
