
/*
 * Definitions of for routines in 'numtypes.h'
 */

// required for 'memcpy()'
#include <cstring>

// === definitions for class 'raw_vector'

template <class T>
raw_vector<T>::raw_vector (const T x, const T y, const T z) {
  v[X] = x; v[Y] = y; v[Z] = z;
}

template <class T>
raw_vector<T>::raw_vector (const raw_vector &a) {
    memcpy ( &v[0], &a.v[0], NDIR * sizeof(T) );
}

template <class T>
raw_vector<T>::raw_vector () {}

template <class T>
const T& raw_vector<T>::operator [] (int i) const { return v[i]; }

template <class T>
T& raw_vector<T>::operator [] (int i) { return v[i]; }

template <class T>
T raw_vector<T>::sqr  () const { return v[X]*v[X] + v[Y]*v[Y] + v[Z]*v[Z]; }

template <class T>
T raw_vector<T>::norm () const { return fabs(v[X]) + fabs(v[Y]) + fabs(v[Z]); }

template <class T>
raw_vector<T> & raw_vector<T>::zero () {
  for (int i=0; i<NDIR; i++) v[i] = 0;
  return *this;
}

template <class T>
raw_vector<T> raw_vector<T>::abs () const {
  raw_vector<T> a;
  for (int i=0; i<NDIR; i++) a.v[i] = fabs(v[i]);
  return a;
}

template <class T>
raw_vector<T> & raw_vector<T>::operator += (const raw_vector<T> &a) {
  for (int i=0; i<NDIR; i++) v[i] += a.v[i];
  return *this;
}

template <class T>
raw_vector<T> & raw_vector<T>::operator -= (const raw_vector<T> &a) {
  for (int i=0; i<NDIR; i++) v[i] -= a.v[i];
  return *this;
}

template <class T>
raw_vector<T> & raw_vector<T>::operator *= (const raw_vector<T> &a) {
  for (int i=0; i<NDIR; i++) v[i] *= a.v[i];
  return *this;
}

template <class T>
raw_vector<T> & raw_vector<T>::operator /= (const raw_vector<T> &a) {
  for (int i=0; i<NDIR; i++) v[i] /= a.v[i];
  return *this;
}

template <class T>
raw_vector<T> & raw_vector<T>::operator += (const real r) {
  for (int i=0; i<NDIR; i++) v[i] += r;
  return *this;
}

template <class T>
raw_vector<T> & raw_vector<T>::operator -= (const real r) {
  for (int i=0; i<NDIR; i++) v[i] -= r;
  return *this;
}

template <class T>
raw_vector<T> & raw_vector<T>::operator *= (const real r) {
  for (int i=0; i<NDIR; i++) v[i] *= r;
  return *this;
}

template <class T>
raw_vector<T> & raw_vector<T>::operator /= (const real r) {
  for (int i=0; i<NDIR; i++) v[i] /= r;
  return *this;
}

template <class T>
raw_vector<T> raw_vector<T>::operator + (const raw_vector<T> &a) const {
  return raw_vector<T>(*this) += a;
}

template <class T>
raw_vector<T> raw_vector<T>::operator - (const raw_vector<T> &a) const {
  return raw_vector<T>(*this) -= a;
}

template <class T>
raw_vector<T> raw_vector<T>::operator * (const real r) const {
  return raw_vector<T>(*this) *= r;
}

template <class T>
raw_vector<T> raw_vector<T>::operator / (const real r) const {
  return raw_vector<T>(*this) /= r;
}

template <class T>
real raw_vector<T>::operator * (const raw_vector<T> &a) const {
  real r=0;
  for (int i=0; i<NDIR; i++) r += v[i] * a.v[i];
  return r;
}

// === various distance functions

real dist (const vector &x1, const vector &x2, const int dir) {
  
  if (dir != -1) return fabs(x2[dir]-x1[dir]);
  
  real dist = fabs(x2[X]-x1[X]);
  
  if (!HAVE_DIR(Y))
    return dist;
  
  else {
    dist = sqr(dist);
    dist += sqr(x2[Y]-x1[Y]);
    if (HAVE_DIR(Z))
      dist += sqr(x2[Z]-x1[Z]);
    return sqrt(dist);
  }
}

real dist_periodic (const vector &x1, const vector &x2, const int dir) {
  
  if (dir != -1) return min( fabs(x2[dir]-x1[dir]), MAX[dir] - fabs(x2[dir]-x1[dir]) );
  
  real dist = min( fabs(x2[X]-x1[X]), MAXX - fabs(x2[X]-x1[X]) );
  
  if (!HAVE_DIR(Y))
    return dist;
  
  else {
    dist = sqr(dist);
    dist += sqr( min( fabs(x2[Y]-x1[Y]), MAXY - fabs(x2[Y]-x1[Y]) ) );
    if (HAVE_DIR(Z))
      dist += sqr( min( fabs(x2[Z]-x1[Z]), MAXZ - fabs(x2[Z]-x1[Z]) ) );
    return sqrt(dist);
  }
}
