
#include "mlmc.h"
#include "multi.h"
#include "balancer.h"

#include <sstream>

Saver::Saver (std::string file_prefix) {
  
  std::ostringstream dfn;
  
  dfn << DATA_PATH << file_prefix << DATA_FILE;
  
  if ( INCLUDE_LEVEL_SUFFIX )
    dfn << "_MESH_LEVEL=" << MESH_LEVEL << "_TYPE=" << TYPE;
  
  if ( MULTIN[L][L] > 1 && ! STATS_FUNCTIONAL)
    dfn << "." << multix << "." << multiy << "." << multiz;
  
  data = new std::ofstream(dfn.str().c_str());
  
  if (multi_main) {
    times = new std::ofstream(TIME_FILE);
    *times << "time = [";
  }
}

Saver::~Saver() {
  
  /*
  data -> close();
  if ( data != NULL )  delete data;
  data = NULL;
  times -> close();
  if ( times != NULL ) delete times;
  times = NULL;
  */
}

void Saver::write_binary (real *r) {
  data -> write ( (char*) r, sizeof(real) );
}

template <class T>
void Saver::write_volume (volume <T> &v, const int &m) {
  int i, j, k;
  for (i = 0; i < v.NXl; i++)
    for (j = 0; j < v.NYl; j++)
      for (k = 0; k < v.NZl; k++)
        #if ! STATS || STATS_SIZE == 1
          this -> write_binary ( & realCast(v[i][j][k]) [m] );
        #else
          for (int c = 0; c < STATS_SIZE; c++)
            this -> write_binary ( & realCast(v[i][j][k][c]) [m] );
        #endif
}

template <class T>
void Saver::write_value (T &v) {
  #if ! STATS || STATS_SIZE == 1
    this -> write_binary (&v);
  #else
    for (int c = 0; c < STATS_SIZE; c++)
      this -> write_binary (&v[c]);
  #endif
}

template <class CT, class XT>
void Saver::snapshot (volume <CT> &CV, volume <XT> &XV) {
  
  if (multi_samplers_root) {
    
    int m;
    
    //  write time of the snapshot
    if (multi_main) {
      *times << halt_times[SNAPSHOT] <<", ";
      times -> flush();
    }
    
    // write ConservedVars
    for (m=0; m<SIZE_CV; m++)
      this -> write_volume (CV, m);
    
    // write XtraVars
    for (m=0; m<SIZE_XV; m++)
      this -> write_volume (XV, m);
    
    data -> flush();
  }
}

template <class T>
void Saver::snapshot (T &v) {
  
  if (multi_samplers_root) {
    
    //  write time of the snapshot
    if (multi_main) {
      *times << halt_times[SNAPSHOT] <<", ";
      times -> flush();
    }
    
    this -> write_value (v);
    data -> flush();
  }
}

void Saver::finalize () {
  
  if (multi_main) {
    *times << "]\n";
    times -> flush();
  }
}

template <class CT, class XT>
void Saver::snapshots (volume <CT> *CV, volume <XT> *XV) {
  for (SNAPSHOT = 0; SNAPSHOT < NSAVES+SAVE0; SNAPSHOT++)
    snapshot (CV[SNAPSHOT], XV[SNAPSHOT]);
  finalize();
}

template <class T>
void Saver::snapshots (T *v) {
  for (SNAPSHOT = 0; SNAPSHOT < NSAVES+SAVE0; SNAPSHOT++)
    snapshot (v[SNAPSHOT]);
  finalize();
}
