
#ifndef SAVER_H
#define SAVER_H

#include <fstream>

// Saver class for saving volumes in a binary format
class Saver {

private:
  
  // pointer to data file for volumes
  std::ofstream *data;
  
  // pointer to data file for time steps
  std::ofstream *times;
  
  // number of 'reals' for each variable
  // (more than one only for histograms) 
  int batch_size;
  
  // write a volume of the specified m-th variable of ConservedVars or XtraVars
  template <class T>
  void write_volume (volume <T> &v, const int &m);
  
  // write the value of the functional
  template <class T>
  void write_value (T &v);
  
  // write in a binary mode to the file
  void write_binary (real *r);
  
public:
  
  // constructor (sets the file name, etc.)
  Saver (const std::string file_prefix="");
  
  // destructor
  ~Saver();
  
  // write one volume snapshot to the file
  template <class CT, class XT>
  void snapshot (volume <CT> &CV, volume <XT> &XV);
  
  // write one functional snapshot to the file
  template <class T>
  void snapshot (T &v);
  
  // finalize saving procedures
  void finalize ();
  
  // iterate over all volume snapshots and finalize
  // i.e. combines 'snapshot()' and 'finalize()'
  template <class CT, class XT>
  void snapshots (volume <CT> *CV, volume <XT> *XV);
  
  // iterate over all functional snapshots and finalize
  // i.e. combines 'snapshot()' and 'finalize()'
  template <class T>
  void snapshots (T *v);
};

// must be here in order to have arbitrary templated objects
#include "saver.cpp"

#endif
