
#include "equation.h"
#include "solver-util-hll.h"
#include "solver.h"

///::title:: HLL solver (standard 2 wave) | for Burgers'
///::depends:: solver-burgers-hll-util
///::consts:: HLL_dt=1

// variables for local wave speeds
real sl, sr, cs;

void solver_calc_flux (ConservedVars &F,
                       const AllVariables &lV, const AllVariables &rV,
                       int i, int j, int k, int d)
{
  if (sl == 0) Zero(F);
  else if (sl > 0) calc_point_flux(F, lV, d);
  else if (sr < 0) calc_point_flux(F, rV, d);
  else {
    ConservedVars lF, rF;

    calc_point_flux(lF, lV, d);
    calc_point_flux(rF, rV, d);
    F = (sr*lF - sl*rF + sr*sl*(rV.U - lV.U))/(sr-sl);
  }
}

real solver_calc_max_speed (const AllVariables &lV, const AllVariables &rV,
                            const int i, const int j, const int k, const int d)
{
  hll_speeds (lV, rV, d, sl, sr, cs);
  return max ( fabs(sl), fabs(sr) );
}
