
#include "equation.h"
#include "solver-util-hll.h"
#include "solver.h"

///::title:: HLL3L solver (Linde 3 wave) | for MHD
///::depends:: solver-mhd-hll-util
///::consts:: HLL_dt=1
///::vars:: POWELL=1

// variables for local wave speeds
real sl, sr, cs;

// vectors for velocities and magnetic fields
vector us, Bs;

void solver_calc_flux(ConservedVars &F,
		      const AllVariables &lV, const AllVariables &rV,
		      int i, int j, int k, int d)
{
  ConservedVars lF, rF;
  calc_point_flux(lF, lV, d);
  calc_point_flux(rF, rV, d);
  ConservedVars Us = (sr*rV.U - sl*lV.U - rF + lF)/(sr - sl);
  us = Us.m / Us.rho;
  Bs = Us.B;
  if (sl == 0) Zero(F);
  else if (sl > 0) F = lF;
  else if (sr < 0) F = rF;
  else {
    real sm = us[d];
    ConservedVars dU = rV.U - lV.U;
    ConservedVars dF = rF - lF;
    real s = (dF - sm*dU).norm() / (dU.norm() + 0.000000001);
    real alpha = max(0, 1-s/(cs + 0.000000001));
    real f;
    if (sm > 0) {
      f = sl*(alpha*sm + (1-alpha)*sr);
      //      Us -= (sr-sm)*dU/(sr-sl);
    } else {
      f = sr*(alpha*sm + (1-alpha)*sl);
      //      Us += (sm-sl)*dU/(sr-sl);
    }
    F = (sr*lF - sl*rF + f*dU)/(sr-sl);
  }
  
  if (POWELL) powell_source (us, Bs, lV, rV, i, j, k, d);
}

real solver_calc_max_speed (const AllVariables &lV, const AllVariables &rV,
                            const int i, const int j, const int k, const int d)
{
  hll_speeds (lV, rV, d, sl, sr, cs);
  return max ( fabs(sl), fabs(sr) );
}
