
#include "equation.h"
#include "solver.h"
#include "solver-util-ec.h"

///::title:: EC solver (Entropy Conservative)
///::consts:: STANDARD_TOPOGRAPHY_QUAD=0
///::depends:: solver-sw-ec-util
///::macro:: PRE_RECONSTRUCTION_HOOK  solver-sw-ec-util.h ConservedVars_to_ReconstVars();
///::macro:: POST_RECONSTRUCTION_HOOK solver-sw-ec-util.h ReconstVars_to_ConservedVars();

void solver_calc_flux(ConservedVars &F,
                      const AllVariables &lV, const AllVariables &rV,
                      int i, int j, int k, int d)
{
  // for EC flux, we get needed ConservedVars from ReconstructionVars U, Xtra
  
  ConservedVars lCV = U[i-DX[d]][j-DY[d]][k];
  ConservedVars rCV = U[i][j][k];
  XtraVars      lXV = Xtra[i-DX[d]][j-DY[d]][k];
  XtraVars      rXV = Xtra[i][j][k];
  
  ReconstVars_to_ConservedVars (lCV, lXV);
  ReconstVars_to_ConservedVars (rCV, rXV);
  
  // === EC flux
  
  ec_flux (F, lCV, rCV, lXV, rXV, d);
}

real solver_calc_max_speed (const AllVariables &lV, const AllVariables &rV,
                            const int i, const int j, const int k, const int d)
{
  return calc_max_speed (U[i][j][k], Xtra[i][j][k], d);
}
