
///::title:: ES solver (Rusanov type, Entropy Stable)
///::consts:: STANDARD_TOPOGRAPHY_QUAD=0
///::depends:: solver-sw-ec-util
///::macro:: PRE_RECONSTRUCTION_HOOK  solver-sw-ec-util.h ConservedVars_to_ReconstVars();
///::macro:: POST_RECONSTRUCTION_HOOK solver-sw-ec-util.h ReconstVars_to_ConservedVars();

#include "equation.h"
#include "solver.h"
#include "solver-util-ec.h"

void solver_calc_flux(ConservedVars &F,
                      const AllVariables &lV, const AllVariables &rV,
                      int i, int j, int k, int d)
{
  // construction of jump in ReconstructionVars
  
  real V [3];
  
  V[0] = jump ( lV.U.h    , rV.U.h     );
  V[1] = jump ( lV.XU.u[X], rV.XU.u[X] );
  V[2] = jump ( lV.XU.u[Y], rV.XU.u[Y] );
  
  // for EC flux and numerical diffusion operator, we get needed ConservedVars from ReconstructionVars U, Xtra
  
  ConservedVars lCV = U[i-DX[d]][j-DY[d]][k];
  ConservedVars rCV = U[i][j][k];
  XtraVars      lXV = Xtra[i-DX[d]][j-DY[d]][k];
  XtraVars      rXV = Xtra[i][j][k];
  
  ReconstVars_to_ConservedVars (lCV, lXV);
  ReconstVars_to_ConservedVars (rCV, rXV);
  
  // === EC flux
  
  ec_flux (F, lCV, rCV, lXV, rXV, d);
  
  // === numerical diffusion
  
  // construction of matrix R
  
  real R [3][3];
  
  compute_R (R, lCV, rCV, lXV, rXV, d);
  
  // construction of diagonal maximal eigenvalue D for Rusanov type diffusion
  
  real D;
  
  real sqrt_gh = sqrt ( GRAVITY * avg(lCV.h, rCV.h) );
  
  D = max ( fabs( avg(lXV.u[d], rXV.u[d]) - sqrt_gh ), fabs( avg(lXV.u[d], rXV.u[d]) + sqrt_gh ) );
  
  // computing numerical diffusion term RDR'V
  
  // R'V
  
  real RV [3] = {0.0};
  
  for (int a=0; a<3; a++)
    for (int b=0; b<3; b++)
      RV[a] += R[b][a] * V[b];
  
  // D(R'V)
  real DRV [3] = {0.0};
  
  for (int a=0; a<3; a++)
    DRV[a] = D * RV[a];
  
  // R(DR'V)
  real RDRV [3] = {0.0};
  
  for (int a=0; a<3; a++)
    for (int b=0; b<3; b++)
      RDRV[a] += R[a][b] * DRV[b];
  
  // adding numerical diffusion to EC flux
  
  F.h     -= 0.5 * RDRV[0];
  F.hu[X] -= 0.5 * RDRV[1];
  F.hu[Y] -= 0.5 * RDRV[2];
}

real solver_calc_max_speed (const AllVariables &lV, const AllVariables &rV,
                            const int i, const int j, const int k, const int d)
{
  return calc_max_speed (U[i][j][k], Xtra[i][j][k], d);
}
