
/*
 * Definitions of 'histogram' struct (static version)
 */

#include "structures.h"

#include <iostream>

template <class T>
histogram<T> & histogram<T>::operator += (const histogram<T> &a) {
  for (int b=0; b<NBINS; b++)
    data[b] += a.data[b];
  return *this;
}

template <class T>
histogram<T> & histogram<T>::operator -= (const histogram<T> &a) {
  for (int b=0; b<NBINS; b++)
    data[b] -= a.data[b];
  return *this;
}

template <class T>
histogram<T> & histogram<T>::operator += (const T &sample) {
  
  int SIZE = sizeof(T) / sizeof(real);
  // for each variable in sample
  for (int m=0; m<SIZE; m++) {
    int b = 0;
    // determine which bin this sample of a variable should go to
    // TODO: perform bisection for search
    while (realCast(sample) [m] > THRESHOLDS[b] && b < NBINS-1)
      b++;
    realCast (data[b]) [m] ++;
  }
  
  return *this;
}

template <class T>
histogram<T> & histogram<T>::operator *= (const real r) {
  for (int b=0; b<NBINS; b++)
    data[b] *= r;
  return *this;
}

template <class T>
histogram<T> & histogram<T>::operator /= (const real r) {
  for (int b=0; b<NBINS; b++)
    data[b] /= r;
  return *this;
}

template <class T>
const T& histogram<T>::operator [] (int i) const { return data[i]; }

template <class T>
T& histogram<T>::operator [] (int i) { return data[i]; }

template <class T>
void Zero (histogram<T> &a) {
  for (int b=0; b<NBINS; b++)
    Zero ( a.data[b] );
}

template <class T>
void NonNegative (histogram<T> &a) {
  for (int b=0; b<NBINS; b++)
    NonNegative ( a.data[b] );
}

void histogram_init () {
  
  THRESHOLDS[0] = HISTOGRAM_MIN;
  WIDTH = (real) ( HISTOGRAM_MAX - HISTOGRAM_MIN ) / (NBINS - 2);
  
  // there must be at least 3 bins
  if ( NBINS < 3 ) {
      std::cerr << " :: NBINS must be at least 3." << std::endl;
      std::exit(1);
  }
  
  for (int b=1; b<NBINS-1; b++)
    THRESHOLDS[b] = THRESHOLDS[b-1] + WIDTH;
}
