
/*
 * Declarations of 'histogram' struct (static version)
 */

#ifndef HISTOGRAMS_H
#define HISTOGRAMS_H

#include "numtypes.h"

// histogram (static version)

template <class T>
struct histogram {
  
  // pointer to actual array of bins
  T data [NBINS];
  
  // operator overloading to add/subtract two histograms
  histogram & operator += (const histogram &a);
  histogram & operator -= (const histogram &a);
  
  // operator overloading to add additional sample value to histogram
  histogram & operator += (const T &sample);
  
  // operator overloading to uniformly scale values of the histogram
  histogram & operator *= (const real r);
  histogram & operator /= (const real r);
  
  // iterators
  const T& operator[] (int i) const;
        T& operator[] (int i);
};

// array of THRESHOLDS for bins
static real THRESHOLDS [NBINS-1];

// width of one bin in histograms
static real WIDTH;

// useful functions
template <class T> void Zero        (histogram<T> &a);
template <class T> void NonNegative (histogram<T> &a);

// THRESHOLDS initialization
static void histogram_init ();

// must be here in order to have arbitrary templated objects and inlining
#include "stats-histograms.cpp"

#endif
