
///::title:: Mean and covariance (stable version, 'online, one pass')
///::consts:: STATS_MEAN_COVAR=1 STATS_COUNT=2 STATS_SIZE=1 STATS_FUNCTIONAL=0
///::vars:: COVAR_X=0 COVAR_Y=0 COVAR_Z=0 COVAR_SOURCE_RANK=0

// TODO: modify everything accordingly!

#include "stats.h"

#if STATS

#include "mlmc.h"
#include "multi.h"
#include "saver.h"

// volumes for MC estimates of mean and covariance (for each snapshot)
// iterators [mesh_level][type] indicate estimate of type 'type' at mesh level 'mesh_level'

volume <ConservedVars> mean_U [L+1][2] [NSAVES+SAVE0];
volume <XtraVars>      mean_X [L+1][2] [NSAVES+SAVE0];

volume <ConservedVars> covariance_U  [L+1][2] [NSAVES+SAVE0];
volume <XtraVars>      covariance_X  [L+1][2] [NSAVES+SAVE0];

void* stats_send_buffer_CV () {
  switch (STAT) {
    case 0: return mean_U       [MESH_LEVEL][TYPE][SNAPSHOT] .data;
    case 1: return covariance_U [MESH_LEVEL][TYPE][SNAPSHOT] .data;
  }
}

void* stats_send_buffer_XV () {
  switch (STAT) {
    case 0: return mean_X       [MESH_LEVEL][TYPE][SNAPSHOT] .data;
    case 1: return covariance_X [MESH_LEVEL][TYPE][SNAPSHOT] .data;
  }
}

// volumes for receive buffers required in parallel implementations

volume <ConservedVars> recv_mean_U;
volume <XtraVars>      recv_mean_X;

volume <ConservedVars> recv_covariance_U;
volume <XtraVars>      recv_covariance_X;

void* stats_recv_buffer_CV () {
  switch (STAT) {
    case 0: return recv_mean_U       .data;
    case 1: return recv_covariance_U .data;
  }
}

void* stats_recv_buffer_XV () {
  switch (STAT) {
    case 0: return recv_mean_X       .data;
    case 1: return recv_covariance_X .data;
  }
}

// WEIGHTS[mesh_level][type]
// number of type ='type' samples computed on the mesh level 'mesh_level' so far
long_int WEIGHTS[L+1][2];

void stats_init () {
  
  WEIGHTS[MESH_LEVEL][TYPE] = 0;
  
  STATS_SEND_RECV[0] = 1;
  STATS_SEND_RECV[1] = 0;
  
  STATS_REDUCE[0] = 0;
  STATS_REDUCE[1] = 1;
}

void stats_volumes_snapshot_level_init () {
  
  mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  
  covariance_U [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  covariance_X [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
}

void stats_recv_volumes_level_init () {
  
  recv_mean_U .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  recv_mean_X .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  
  recv_covariance_U .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  recv_covariance_X .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
}

void stats_copy () {
  
  mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_U;
  mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_X;
  
  covariance_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_covariance_U;
  covariance_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_covariance_X;
}

// numerically stable covariance computation algorithm by Chan et al.
template<class T>
void covariance_update (const T &mean, T &covariance, const T &sample, const long_int &i, const T &diff) {
  
  if ( i==1 ) return;
  
  T delta (sample);
  delta -= mean;
  //delta .multiply_elementwise ( (1 - 1.0/i) * (sample[0][0][0] - mean[0][0][0]) );
  delta .multiply_elementwise ( (1 - 1.0/i) * diff );
  covariance += delta;
}

// numerically stable mean computation algorithm by Chan et al.
template<class T>
void mean_update (T &mean, const T &sample, const long_int &i) {
  T delta (sample);
  delta -= mean;
  delta /= i;
  mean += delta;
}

void stats_update () {
  
  // update WEIGHTS (during the FIRST snapshot)
  if ( SNAPSHOT == FIRST_SNAPSHOT )
    WEIGHTS[MESH_LEVEL][TYPE]++;
  
  // broadcast the value of (U - mean_U) at the required position
  ConservedVars diff_CV = multi_broadcast_CV ( U [COVAR_X][COVAR_Y][COVAR_Z] - mean_U [MESH_LEVEL][TYPE][SNAPSHOT] [COVAR_X][COVAR_Y][COVAR_Z], COVAR_SOURCE_RANK );
  
  // update mean and covariance for ConservedVars
  
  covariance_update (mean_U[MESH_LEVEL][TYPE][SNAPSHOT], covariance_U[MESH_LEVEL][TYPE][SNAPSHOT], U, WEIGHTS[MESH_LEVEL][TYPE], diff_CV);
  
  mean_update (mean_U[MESH_LEVEL][TYPE][SNAPSHOT], U, WEIGHTS[MESH_LEVEL][TYPE]);
  
  // broadcast the value of (Xtra - mean_X) at the required position
  XtraVars diff_XV = multi_broadcast_XV ( Xtra [COVAR_X][COVAR_Y][COVAR_Z] - mean_X [MESH_LEVEL][TYPE][SNAPSHOT] [COVAR_X][COVAR_Y][COVAR_Z], COVAR_SOURCE_RANK );
  
  // update mean and covariance for XtraVars
  
  covariance_update (mean_X[MESH_LEVEL][TYPE][SNAPSHOT], covariance_X[MESH_LEVEL][TYPE][SNAPSHOT], Xtra, WEIGHTS[MESH_LEVEL][TYPE], diff_XV);
  
  mean_update (mean_X[MESH_LEVEL][TYPE][SNAPSHOT], Xtra, WEIGHTS[MESH_LEVEL][TYPE]);
}

void stats_finalize_updates () {}

// numerically stable covariance combination algorithm by Chan et al.
// combined covariance is stored in 'covar1'
// for the complete combination, second covariance 'covar2' should be added to 'covar1'
// i.e. STATS_REDUCE[1] = 1 is assumed
template<class T>
void covariance_combine (const T &mean1, T &covar1, const real &w1, const T &mean2, const real &w2, const T &diff) {
  
  if ( w1 == 0 || w2 == 0 ) return;
  
  T delta (mean2);
  delta -= mean1;
  //delta .multiply_elementwise ( (w1 * w2) / (w1 + w2) * (mean1[0][0][0] - mean2[0][0][0]) );
  delta .multiply_elementwise ( (w1 * w2) / (w1 + w2) * diff );
  covar1 += delta;
}

// numerically stable mean computation algorithm by Chan et al.
// combined mean is stored in 'mean1'
template<class T>
void mean_combine (T &mean1, const real &w1, const T &mean2, const real &w2) {
  
  // if w2 is much smaller than w1
  if ( w2 < 0.01 * w1 ) {
    T delta (mean2);
    delta -= mean1;
    delta *= w2 / (w1 + w2);
    mean1 += delta;
  }
  
  // if w1 ~ w2 and both large
  else {
    mean1 *= w1 / (w1 + w2);
    mean1 += mean2 * ( w2 / (w1 + w2) );
  }
}

void stats_combine (const real weight) {
  
  // broadcast the value of (mean_U - recv_mean_U) at the required position
  ConservedVars diff_CV = multi_broadcast_CV ( mean_U [MESH_LEVEL][TYPE][SNAPSHOT] [COVAR_X][COVAR_Y][COVAR_Z] - recv_mean_U [COVAR_X][COVAR_Y][COVAR_Z], COVAR_SOURCE_RANK );
  
  // combine mean and covariance for ConservedVars
  
  covariance_combine (mean_U[MESH_LEVEL][TYPE][SNAPSHOT], covariance_U[MESH_LEVEL][TYPE][SNAPSHOT], WEIGHTS[MESH_LEVEL][TYPE], recv_mean_U, weight, diff_CV);
  
  mean_combine (mean_U[MESH_LEVEL][TYPE][SNAPSHOT], WEIGHTS[MESH_LEVEL][TYPE], recv_mean_U, weight);
  
  // broadcast the value of (mean_X - recv_mean_X) at the required position
  ConservedVars diff_XV = multi_broadcast_XV ( mean_X [MESH_LEVEL][TYPE][SNAPSHOT] [COVAR_X][COVAR_Y][COVAR_Z] - recv_mean_X [COVAR_X][COVAR_Y][COVAR_Z], COVAR_SOURCE_RANK );
  
  // combine mean and covariance for XtraVars
  
  covariance_combine (mean_X[MESH_LEVEL][TYPE][SNAPSHOT], covariance_X[MESH_LEVEL][TYPE][SNAPSHOT], WEIGHTS[MESH_LEVEL][TYPE], recv_mean_X, weight, diff_XV);
  
  mean_combine (mean_X[MESH_LEVEL][TYPE][SNAPSHOT], WEIGHTS[MESH_LEVEL][TYPE], recv_mean_X, weight);
  
  // update the total number of samples (during the LAST snapshot)
  if ( SNAPSHOT == LAST_SNAPSHOT )
    WEIGHTS[MESH_LEVEL][TYPE] += weight;
}

void stats_reduce () {
  switch (STAT) {
    case 0:
      mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_U;
      mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_X;
    case 1:
      covariance_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_covariance_U;
      covariance_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_covariance_X;
  }
}

void stats_finalize_combines () {
  
  // computing covariance
  if (NM[SAMPLES_LEVEL()] > 1) covariance_U [MESH_LEVEL][TYPE] [SNAPSHOT] /= NM[SAMPLES_LEVEL()];
  if (NM[SAMPLES_LEVEL()] > 1) covariance_X [MESH_LEVEL][TYPE] [SNAPSHOT] /= NM[SAMPLES_LEVEL()];
}

void stats_assemble_mlmc () {
  
  // assemble MLMC estimate of mean
  stats_assemble_mlmc (mean_U);
  stats_assemble_mlmc (mean_X);
  
  // assemble MLMC estimate of covariance
  stats_assemble_mlmc (covariance_U);
  stats_assemble_mlmc (covariance_X);
}

void stats_apply_constraints () {
  
  // apply generic constraints to mean
  for (int i = 0; i < mean_U[L][0][SNAPSHOT].NXl; i++)
    for (int j = 0; j < mean_U[L][0][SNAPSHOT].NYl; j++)
      for (int k = 0; k < mean_U[L][0][SNAPSHOT].NZl; k++)
        apply_constraints (mean_U[L][0][SNAPSHOT] [i][j][k], mean_X[L][0][SNAPSHOT] [i][j][k]);
}

void stats_write_datafiles () {
  
  Saver saver_mean ("");
  saver_mean.snapshots (&mean_U[MESH_LEVEL][TYPE][0], &mean_X[MESH_LEVEL][TYPE][0]);
  
  Saver saver_var ("var_");
  saver_var.snapshots (&covariance_U[MESH_LEVEL][TYPE][0], &covariance_X[MESH_LEVEL][TYPE][0]);
}

#endif
