
///::title:: Mean and kurtosis (stable version, Terriberry)
///::consts:: STATS_MEAN_KURTOSIS=1 STATS_COUNT=2 STATS_SIZE=1 STATS_FUNCTIONAL=0
///::vars:: KURTOSIS_FIX=1

#include "stats.h"

#if STATS

#include "mlmc.h"
#include "multi.h"
#include "saver.h"

// volumes for MC estimates of mean and kurtosis (for each snapshot)
// iterators [mesh_level][type] indicate estimate of type 'type' at mesh level 'mesh_level'

volume <ConservedVars> mean_U [L+1][2] [NSAVES+SAVE0];
volume <XtraVars>      mean_X [L+1][2] [NSAVES+SAVE0];

volume <ConservedVars> kurtosis_U  [L+1][2] [NSAVES+SAVE0];
volume <XtraVars>      kurtosis_X  [L+1][2] [NSAVES+SAVE0];

void* stats_send_buffer_CV () {
  switch (STAT) {
    case 0: return mean_U     [MESH_LEVEL][TYPE][SNAPSHOT] .data;
    case 1: return kurtosis_U [MESH_LEVEL][TYPE][SNAPSHOT] .data;
  }
}

void* stats_send_buffer_XV () {
  switch (STAT) {
    case 0: return mean_X     [MESH_LEVEL][TYPE][SNAPSHOT] .data;
    case 1: return kurtosis_X [MESH_LEVEL][TYPE][SNAPSHOT] .data;
  }
}

// volumes for receive buffers required in parallel implementations

volume <ConservedVars> recv_mean_U;
volume <XtraVars>      recv_mean_X;

volume <ConservedVars> recv_kurtosis_U;
volume <XtraVars>      recv_kurtosis_X;

void* stats_recv_buffer_CV () {
  switch (STAT) {
    case 0: return recv_mean_U     .data;
    case 1: return recv_kurtosis_U .data;
  }
}

void* stats_recv_buffer_XV () {
  switch (STAT) {
    case 0: return recv_mean_X     .data;
    case 1: return recv_kurtosis_X .data;
  }
}

// WEIGHTS[mesh_level][type]
// number of type ='type' samples computed on the mesh level 'mesh_level' so far
long_int WEIGHTS[L+1][2];

void stats_init () {
  
  WEIGHTS[MESH_LEVEL][TYPE] = 0;
  
  STATS_SEND_RECV[0] = 1;
  STATS_SEND_RECV[1] = 0;
  
  STATS_REDUCE[0] = 0;
  STATS_REDUCE[1] = 1;
}

void stats_volumes_snapshot_level_init () {
  
  mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  
  kurtosis_U  [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  kurtosis_X  [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
}

void stats_recv_volumes_level_init () {
  
  recv_mean_U .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  recv_mean_X .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  
  recv_kurtosis_U .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  recv_kurtosis_X .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
}

void stats_copy () {
  
  mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_U;
  mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_X;
  
  kurtosis_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_kurtosis_U;
  kurtosis_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_kurtosis_X;
}

// numerically stable kurtosis computation algorithm by Terriberry
// updates mean as well
template<class T>
void kurtosis_update (const T &mean, T &kurtosis, const T &sample, const long_int &i) {
  
  // TODO: FIX THIS!!! (i.e. M4[s], etc)
  
  // update kurtosis
  
  T delta_n  (sample.NXl, sample.NYl, sample.NZl);
  T delta_n2 (sample.NXl, sample.NYl, sample.NZl);
  T term1    (sample.NXl, sample.NYl, sample.NZl);
  
  T delta (sample);
  delta -= mean;
  
  delta_n += delta;
  delta_n /= i;
  
  delta_n2 += delta_n;
  delta_n2 *= delta_n;
  
  term1 += delta;
  term1 *= delta_n;
  term1 *= i-1;
  
  mean += delta_n;
  
  M4[s] += term1 * delta_n2 * (i*i - 3*i + 3);
  M4[s] += delta_n2 * M2[s] * 6;
  M4[s] -= delta_n * M3[s] * 4;
  
  M3[s] += term1 * delta_n * (i-2);
  M3[s] -= delta_n * M2[s] * 3;
  
  M2[s] += term1;
}

void stats_update () {
  
  // update WEIGHTS (during the FIRST snapshot)
  if ( SNAPSHOT == FIRST_SNAPSHOT )
    WEIGHTS[MESH_LEVEL][TYPE]++;
  
  // update mean and kurtosis for ConservedVars
  
  kurtosis_update (mean_U_l[MESH_LEVEL][TYPE][SNAPSHOT], kurtosis_U_l[MESH_LEVEL][TYPE][SNAPSHOT], U, WEIGHTS[MESH_LEVEL][TYPE]);
  
  // update mean and kurtosis for XtraVars
  
  kurtosis_update (mean_X_l[MESH_LEVEL][TYPE][SNAPSHOT], kurtosis_X_l[MESH_LEVEL][TYPE][SNAPSHOT], Xtra, WEIGHTS[MESH_LEVEL][TYPE]);
}

void stats_finalize_updates () {
  kurtosis_U [MESH_LEVEL][TYPE] [SNAPSHOT] += M4[SNAPSHOT];
  // TODO: shoudn't be division by  NM - 3 ? use if statement?
  kurtosis_U [MESH_LEVEL][TYPE] [SNAPSHOT] /= NM[SAMPLES_LEVEL()] - 1;
}

void stats_combine (const real weight) {
  // TODO: error message: NOT IMPLEMENTED! (for parallel runs)
  
  // update the total number of samples (during the LAST snapshot)
  if ( SNAPSHOT == LAST_SNAPSHOT )
    WEIGHTS[MESH_LEVEL][TYPE] += weight;
}

void stats_reduce () {
  switch (STAT) {
    case 0:
      mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_U;
      mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_X;
    case 1:
      kurtosis_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_kurtosis_U;
      kurtosis_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_kurtosis_X;
  }
}

void stats_finalize_combines () {}

void stats_assemble_mlmc () {
  
  // assemble MLMC estimate of mean
  stats_assemble_mlmc (mean_U);
  stats_assemble_mlmc (mean_X);
  
  // assemble MLMC estimate of variance
  stats_assemble_mlmc (kurtosis_U);
  stats_assemble_mlmc (kurtosis_X);
}

void stats_apply_constraints () {
  
  // apply generic constraints to mean
  for (int i = 0; i < mean_U[L][0][SNAPSHOT].NXl; i++)
    for (int j = 0; j < mean_U[L][0][SNAPSHOT].NYl; j++)
      for (int k = 0; k < mean_U[L][0][SNAPSHOT].NZl; k++)
        apply_constraints (mean_U[L][0][SNAPSHOT] [i][j][k], mean_X[L][0][SNAPSHOT] [i][j][k]);
  
  // kurtosis should be non-negative
  #if KURTOSIS_FIX
  kurtosis_U [L][0][SNAPSHOT] .nonnegative();
  kurtosis_X [L][0][SNAPSHOT] .nonnegative();
  #endif
}

void stats_write_datafiles () {
  
  // MLMC mean estimates are in mean_U[L][0], mean_X[L][0]
  Saver saver_mean ("");
  saver_mean.snapshots (&mean_U[L][0][0], &mean_X[L][0][0]);
  
  // MLMC variance estimates are in kurtosis_U[L][0], kurtosis_X[L][0]
  Saver saver_var ("kurtosis_");
  saver_var.snapshots (&kurtosis_U[L][0][0], &kurtosis_X[L][0][0]);
}

#endif
