
///::title:: Mean and variance (naive unstable version)
///::consts:: STATS_MEAN_VAR_UNSTABLE=1 STATS_COUNT=2 STATS_SIZE=1 STATS_FUNCTIONAL=0
///::vars:: VARIANCE_FIX=1

#include "stats.h"

#if STATS

#include "mlmc.h"
#include "multi.h"
#include "saver.h"

// volumes for MC estimates of mean and variance (for each snapshot)
// iterators [mesh_level][type] indicate estimate of type 'type' at mesh level 'mesh_level'

volume <ConservedVars> mean_U [L+1][2] [NSAVES+SAVE0];
volume <XtraVars>      mean_X [L+1][2] [NSAVES+SAVE0];

volume <ConservedVars> variance_U  [L+1][2] [NSAVES+SAVE0];
volume <XtraVars>      variance_X  [L+1][2] [NSAVES+SAVE0];

void* stats_send_buffer_CV () {
  switch (STAT) {
    case 0: return mean_U     [MESH_LEVEL][TYPE][SNAPSHOT] .data;
    case 1: return variance_U [MESH_LEVEL][TYPE][SNAPSHOT] .data;
  }
}

void* stats_send_buffer_XV () {
  switch (STAT) {
    case 0: return mean_X     [MESH_LEVEL][TYPE][SNAPSHOT] .data;
    case 1: return variance_X [MESH_LEVEL][TYPE][SNAPSHOT] .data;
  }
}

// volumes for receive buffers required in parallel implementations

volume <ConservedVars> recv_mean_U;
volume <XtraVars>      recv_mean_X;

volume <ConservedVars> recv_variance_U;
volume <XtraVars>      recv_variance_X;

void* stats_recv_buffer_CV () {
  switch (STAT) {
    case 0: return recv_mean_U     .data;
    case 1: return recv_variance_U .data;
  }
}

void* stats_recv_buffer_XV () {
  switch (STAT) {
    case 0: return recv_mean_X     .data;
    case 1: return recv_variance_X .data;
  }
}

void stats_init () {
  
  STATS_SEND_RECV[0] = 0;
  STATS_SEND_RECV[1] = 0;
  
  STATS_REDUCE[0] = 1;
  STATS_REDUCE[1] = 1;
}

void stats_volumes_snapshot_level_init () {
  
  mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  
  variance_U [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  variance_X [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
}

void stats_recv_volumes_level_init () {
  
  recv_mean_U .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  recv_mean_X .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  
  recv_variance_U .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  recv_variance_X .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
}

void stats_copy () {
  
  mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_U;
  mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_X;
  
  variance_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_variance_U;
  variance_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_variance_X;
}

// naive numerically unstable variance computation algorithm
template<class T>
void variance_update (T &variance, const T &sample) {
  T tmp (sample);
  variance += tmp.sqr();
}

// mean computation algorithm
template<class T>
void mean_update (T &mean, const T &sample) {
  mean += sample;
}

void stats_update () {
  
  // update mean and variance for ConservedVars
  
  variance_update (variance_U[MESH_LEVEL][TYPE][SNAPSHOT], U);
  
  mean_update (mean_U[MESH_LEVEL][TYPE][SNAPSHOT], U);
  
  // update mean and variance for XtraVars
  
  variance_update (variance_X[MESH_LEVEL][TYPE][SNAPSHOT], Xtra);
  
  mean_update (mean_X[MESH_LEVEL][TYPE][SNAPSHOT], Xtra);
}

void stats_finalize_updates () {}

void stats_combine (const real weight) {
  // STATS_REDUCE[0] = 1 and STATS_REDUCE[1] = 1 are already sufficient
}

void stats_reduce () {
  switch (STAT) {
    case 0:
      mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_U;
      mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_X;
    case 1:
      variance_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_variance_U;
      variance_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_variance_X;
  }
}

void stats_finalize_combines () {
  
  // computing variance
  
  variance_U [MESH_LEVEL][TYPE] [SNAPSHOT] -= mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] * mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] / NM[SAMPLES_LEVEL()];
  variance_X [MESH_LEVEL][TYPE] [SNAPSHOT] -= mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] * mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] / NM[SAMPLES_LEVEL()];
  
  if (NM[SAMPLES_LEVEL()] > 2) variance_U [MESH_LEVEL][TYPE] [SNAPSHOT] /= NM[SAMPLES_LEVEL()] - 1;
  if (NM[SAMPLES_LEVEL()] > 2) variance_X [MESH_LEVEL][TYPE] [SNAPSHOT] /= NM[SAMPLES_LEVEL()] - 1;
  
  // computing mean
  if (NM[SAMPLES_LEVEL()] > 1) mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] /= NM[SAMPLES_LEVEL()];
  if (NM[SAMPLES_LEVEL()] > 1) mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] /= NM[SAMPLES_LEVEL()];
}

void stats_assemble_mlmc () {
  
  // assemble MLMC estimate of mean
  stats_assemble_mlmc (mean_U);
  stats_assemble_mlmc (mean_X);
  
  // assemble MLMC estimate of variance
  stats_assemble_mlmc (variance_U);
  stats_assemble_mlmc (variance_X);
}

void stats_apply_constraints () {
  
  // apply generic constraints to mean
  for (int i = 0; i < mean_U[L][0][SNAPSHOT].NXl; i++)
    for (int j = 0; j < mean_U[L][0][SNAPSHOT].NYl; j++)
      for (int k = 0; k < mean_U[L][0][SNAPSHOT].NZl; k++)
        apply_constraints (mean_U[L][0][SNAPSHOT] [i][j][k], mean_X[L][0][SNAPSHOT] [i][j][k]);
  
  // variance should be non-negative
  #if VARIANCE_FIX
  variance_U [L][0][SNAPSHOT] .nonnegative();
  variance_X [L][0][SNAPSHOT] .nonnegative();
  #endif
}

void stats_write_datafiles () {
  
  // MLMC mean estimates are in mean_U[L][0], mean_X[L][0]
  Saver saver_mean("");
  saver_mean.snapshots (&mean_U[L][0][0], &mean_X[L][0][0]);
  
  // MLMC variance estimates are in var_U[L][0], var_X[L][0]
  Saver saver_var("var_");
  saver_var.snapshots (&variance_U[L][0][0], &variance_X[L][0][0]);
}

#endif
