
///::title:: Mean (naive unstable version for FT-implementation)
///::consts:: STATS_MEAN=1 STATS_COUNT=1 STATS_SIZE=1 STATS_FUNCTIONAL=0

#include "stats.h"

#if STATS

#include "mlmc.h"
#include "multi.h"
#include "saver.h"

// volumes for MC estimates of mean (for each snapshot)
// iterators [mesh_level][type] indicate estimate of type 'type' at mesh level 'mesh_level'

volume <ConservedVars> mean_U [L+1][2] [NSAVES+SAVE0];
volume <XtraVars>      mean_X [L+1][2] [NSAVES+SAVE0];

void* stats_send_buffer_CV () {
  return mean_U [MESH_LEVEL][TYPE][SNAPSHOT] .data;
}

void* stats_send_buffer_XV () {
  return mean_X [MESH_LEVEL][TYPE][SNAPSHOT] .data;
}

// volumes for receive buffers required in parallel implementations

volume <ConservedVars> recv_mean_U;
volume <XtraVars>      recv_mean_X;

void* stats_recv_buffer_CV () {
  return recv_mean_U.data;
}

void* stats_recv_buffer_XV () {
  return recv_mean_X.data;
}

void stats_init () {
  STATS_SEND_RECV[0] = 0;
  STATS_REDUCE[0] = 1;
}

void stats_volumes_snapshot_level_init () {
  mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
}

void stats_recv_volumes_level_init () {  
  recv_mean_U .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  recv_mean_X .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
}

void stats_copy () {
  mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_U;
  mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_X;
}

// mean computation algorithm
template<class T>
void mean_update (T &mean, const T &sample) {
  mean += sample;
}

void stats_update () {
  
  // update mean for ConservedVars
  
  mean_update (mean_U[MESH_LEVEL][TYPE][SNAPSHOT], U);
  
  // update mean for XtraVars
  
  mean_update (mean_X[MESH_LEVEL][TYPE][SNAPSHOT], Xtra);
}

void stats_finalize_updates () {}

void stats_combine (const real weight) {}

void stats_reduce () {
  mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_U;
  mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_mean_X;
}

void stats_finalize_combines () {
  // computing mean
  if (NM[SAMPLES_LEVEL()] > 1) mean_U [MESH_LEVEL][TYPE] [SNAPSHOT] /= NM[SAMPLES_LEVEL()];
  if (NM[SAMPLES_LEVEL()] > 1) mean_X [MESH_LEVEL][TYPE] [SNAPSHOT] /= NM[SAMPLES_LEVEL()];
}

void stats_assemble_mlmc () {
  // assemble MLMC estimate of mean
  stats_assemble_mlmc (mean_U);
  stats_assemble_mlmc (mean_X);
}

void stats_apply_constraints () {
  
  // apply generic constraints to mean
  for (int i = 0; i < mean_U[L][0][SNAPSHOT].NXl; i++)
    for (int j = 0; j < mean_U[L][0][SNAPSHOT].NYl; j++)
      for (int k = 0; k < mean_U[L][0][SNAPSHOT].NZl; k++)
        apply_constraints (mean_U[L][0][SNAPSHOT] [i][j][k], mean_X[L][0][SNAPSHOT] [i][j][k]);
}

void stats_write_datafiles () {
  
  // MLMC mean estimates are in mean_U[L][0], mean_X[L][0]
  Saver saver ("");
  saver.snapshots (&mean_U[L][0][0], &mean_X[L][0][0]);
}

#endif
