
///::title:: Threshold (for Shallow Water equations)
///::vars:: WATER_LEVEL_THRESHOLD=1.002
///::consts:: STATS_THRESHOLD=1 STATS_COUNT=1 STATS_SIZE=1 STATS_FUNCTIONAL=0

#include "stats.h"

#if STATS

#include "mlmc.h"
#include "multi.h"
#include "saver.h"

// volumes for MC estimates of mean (for each snapshot)
// iterators [mesh_level][type] indicate estimate of type 'type' at mesh level 'mesh_level'

volume <ConservedVars> threshold_U [L+1][2] [NSAVES+SAVE0];
volume <XtraVars>      threshold_X [L+1][2] [NSAVES+SAVE0];

void* stats_send_buffer_CV () {
  return threshold_U [MESH_LEVEL][TYPE][SNAPSHOT] .data;
}

void* stats_send_buffer_XV () {
  return threshold_X [MESH_LEVEL][TYPE][SNAPSHOT] .data;
}

// volumes for receive buffers required in parallel implementations

volume <ConservedVars> recv_threshold_U;
volume <XtraVars>      recv_threshold_X;

void* stats_recv_buffer_CV () {
  return recv_threshold_U.data;
}

void* stats_recv_buffer_XV () {
  return recv_threshold_X.data;
}

void stats_init () {
  STATS_SEND_RECV[0] = 0;
  STATS_REDUCE[0] = 1;
}

void stats_volumes_snapshot_level_init () {
  threshold_U [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  threshold_X [MESH_LEVEL][TYPE] [SNAPSHOT] .reinit (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
}

void stats_recv_volumes_level_init () {  
  recv_threshold_U .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
  recv_threshold_X .resize (max(1, N[X][MESH_LEVEL] / MULTIXl), max(1, N[Y][MESH_LEVEL] / MULTIYl), max(1, N[Z][MESH_LEVEL] / MULTIZl), 0);
}

void stats_copy () {
  threshold_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_threshold_U;
  threshold_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_threshold_X;
}

void stats_update () {
  // check whether water level h+b exceeds WATER_LEVEL_THRESHOLD
  for (int i = 0; i < NXl; i++)
    for (int j = 0; j < NYl; j++)
      threshold_U[MESH_LEVEL][TYPE][SNAPSHOT][i][j][0].h += (U[i][j][0].h + Xtra[i][j][0].b > WATER_LEVEL_THRESHOLD) ? 1.0 : 0.0;
}

void stats_finalize_updates () {}

void stats_combine (const real weight) {
  // STATS_REDUCE[0] = 1 is already sufficient
}

void stats_reduce () {
  threshold_U [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_threshold_U;
  threshold_X [MESH_LEVEL][TYPE] [SNAPSHOT] += recv_threshold_X;
}

void stats_finalize_combines () {
  // computing mean
  if (NM[SAMPLES_LEVEL()] > 1) threshold_U [MESH_LEVEL][TYPE] [SNAPSHOT] /= NM[SAMPLES_LEVEL()];
}

void stats_apply_constraints () {
  
  // probability of exceeding threshold is always in [0,1]
  for (int i = 0; i < threshold_U[L][0][SNAPSHOT].NXl; i++)
    for (int j = 0; j < threshold_U[L][0][SNAPSHOT].NYl; j++)
      for (int k = 0; k < threshold_U[L][0][SNAPSHOT].NZl; k++) {
        if ( threshold_U[L][0][SNAPSHOT] [i][j][k] .h > 1.0 )
          threshold_U[L][0][SNAPSHOT] [i][j][k] .h = 1.0;
        if ( threshold_U[L][0][SNAPSHOT] [i][j][k] .h < 0.0 )
          threshold_U[L][0][SNAPSHOT] [i][j][k] .h = 0.0;
      }
}

void stats_assemble_mlmc () {
  stats_assemble_mlmc (threshold_U);
  stats_assemble_mlmc (threshold_X);
}

void stats_write_datafiles () {
  // MLMC mean estimates are in threshold_U[L][0], threshold_X[L][0]
  Saver saver ("threshold_");
  saver_mean.snapshots (&threshold_U[L][0][0], &threshold_X[L][0][0]);
}

#endif
