
#include "structures.h"

// === ConservedVars

ConservedVars operator + (const ConservedVars &a, const ConservedVars &b) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = realCast(a) [m] + realCast(b) [m];
  return v;
}

ConservedVars operator - (const ConservedVars &a, const ConservedVars &b) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = realCast(a) [m] - realCast(b) [m];
  return v;
}

ConservedVars operator * (const ConservedVars &a, const ConservedVars &b) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = realCast(a) [m] * realCast(b) [m];
  return v;
}

ConservedVars operator / (const ConservedVars &a, const ConservedVars &b) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = realCast(a) [m] / realCast(b) [m];
  return v;
}

ConservedVars operator * (real r, const ConservedVars &a) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = r * realCast(a) [m];
  return v;
}

ConservedVars operator * (const ConservedVars &a, real r) { return r*a; }

ConservedVars operator / (const ConservedVars &a, real r) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = realCast(a) [m] / r;
  return v;
}

void Zero(ConservedVars &CV) {
  for (int m=0; m<SIZE_CV; m++)
    realCast(CV) [m] = 0.0;
}

void NonNegative(ConservedVars &CV) {
  for (int m=0; m<SIZE_CV; m++)
    if (realCast(CV) [m] < 0.0) realCast(CV) [m] = 0.0;
}

void Exp(ConservedVars &CV) {
  for (int m=0; m<SIZE_CV; m++)
    realCast(CV) [m] = exp( realCast(CV) [m] );
}

void Log(ConservedVars &CV) {
  for (int m=0; m<SIZE_CV; m++)
    realCast(CV) [m] = log( realCast(CV) [m] );
}

// === XtraVars

XtraVars operator + (const XtraVars &a, const XtraVars &b) {
  XtraVars v;
  for (int m=0; m<SIZE_XV; m++)
    realCast(v) [m] = realCast(a) [m] + realCast(b) [m];
  return v;
}

XtraVars operator - (const XtraVars &a, const XtraVars &b) {
  XtraVars v;
  for (int m=0; m<SIZE_XV; m++)
    realCast(v) [m] = realCast(a) [m] - realCast(b) [m];
  return v;
}

XtraVars operator * (const XtraVars &a, const XtraVars &b) {
  XtraVars v;
  for (int m=0; m<SIZE_XV; m++)
    realCast(v) [m] = realCast(a) [m] * realCast(b) [m];
  return v;
}

XtraVars operator / (const XtraVars &a, const XtraVars &b) {
  XtraVars v;
  for (int m=0; m<SIZE_XV; m++)
    realCast(v) [m] = realCast(a) [m] / realCast(b) [m];
  return v;
}

XtraVars operator * (real r, const XtraVars &a) {
  XtraVars v;
  for (int m=0; m<SIZE_XV; m++)
    realCast(v) [m] = r * realCast(a) [m] / r;
  return v;
}

XtraVars operator * (const XtraVars &a, real r) { return r*a; }

XtraVars operator / (const XtraVars &a, real r) {
  XtraVars v;
  for (int m=0; m<SIZE_XV; m++)
    realCast(v) [m] = realCast(a) [m] / r;
  return v;
}

void Zero(XtraVars &XV) {
  for (int m=0; m<SIZE_XV; m++)
    realCast(XV) [m] = 0.0;
}

void NonNegative(XtraVars &XV) {
  for (int m=0; m<SIZE_XV; m++)
    if (realCast(XV) [m] < 0.0) realCast(XV) [m] = 0.0;
}

void Exp(XtraVars &XV) {
  for (int m=0; m<SIZE_XV; m++)
    realCast(XV) [m] = exp( realCast(XV) [m] );
}

void Log(XtraVars &XV) {
  for (int m=0; m<SIZE_XV; m++)
    realCast(XV) [m] = log( realCast(XV) [m] );
}

// === PrimitiveVars

PrimitiveVars operator + (const PrimitiveVars &a, const PrimitiveVars &b)
{
  PrimitiveVars v;
  for (int m=0; m<SIZE_PV; m++)
    realCast(v) [m] = realCast(a) [m] + realCast(b) [m];
  return v;
}

PrimitiveVars operator - (const PrimitiveVars &a, const PrimitiveVars &b)
{
  PrimitiveVars v;
  for (int m=0; m<SIZE_PV; m++)
    realCast(v) [m] = realCast(a) [m] - realCast(b) [m];
  return v;
}

PrimitiveVars operator * (const PrimitiveVars &a, const PrimitiveVars &b)
{
  PrimitiveVars v;
  for (int m=0; m<SIZE_PV; m++)
    realCast(v) [m] = realCast(a) [m] * realCast(b) [m];
  return v;
}

PrimitiveVars operator / (const PrimitiveVars &a, const PrimitiveVars &b)
{
  PrimitiveVars v;
  for (int m=0; m<SIZE_PV; m++)
    realCast(v) [m] = realCast(a) [m] / realCast(b) [m];
  return v;
}

PrimitiveVars operator * (real r, const PrimitiveVars &a)
{
  PrimitiveVars v;
  for (int m=0; m<SIZE_PV; m++)
    realCast(v) [m] = r * realCast(a) [m];
  return v;
}

PrimitiveVars operator * (const PrimitiveVars &a, real r) { return r*a; }

PrimitiveVars operator / (const PrimitiveVars &a, real r)
{
  PrimitiveVars v;
  for (int m=0; m<SIZE_PV; m++)
    realCast(v) [m] = realCast(a) [m] / r;
  return v;
}

void Zero(PrimitiveVars &PV) {
  for (int m=0; m<SIZE_PV; m++)
    realCast(PV) [m] = 0.0;
}

// === AllVariables

AllVariables operator + (const AllVariables &a, const AllVariables &b) {
  return AllVariables (a.U + b.U, a.XU + b.XU);
}

AllVariables operator - (const AllVariables &a, const AllVariables &b) {
  return AllVariables (a.U - b.U, a.XU - b.XU);
}

AllVariables operator * (const AllVariables &a, const AllVariables &b) {
  return AllVariables (a.U * b.U, a.XU * b.XU);
}

AllVariables operator / (const AllVariables &a, const AllVariables &b) {
  return AllVariables (a.U / b.U, a.XU / b.XU);
}

AllVariables operator * (const real r, const AllVariables &a) {
  return AllVariables (r * a.U, r * a.XU);
}

AllVariables operator * (const AllVariables &a, const real r) { return r*a; }

AllVariables operator / (const AllVariables &a, const real r) {
  return AllVariables (a.U / r, a.XU / r);
}
