
/*
 * Operator overloading for ConservedVars, PrimitiveVars and XtraVars
 */

#ifndef STRUCTURES_H
#define STRUCTURES_H

// structures for ConservedVars, PrimitiveVars and XtraVars
#include "include.h"

// structure sizes: needed for operator overloading below
static const int SIZE_CV = sizeof(ConservedVars) / sizeof(real);
static const int SIZE_XV = sizeof(XtraVars)      / sizeof(real);
static const int SIZE_PV = sizeof(PrimitiveVars) / sizeof(real);

// === ConservedVars

ConservedVars operator + (const ConservedVars &a, const ConservedVars &b);
ConservedVars operator - (const ConservedVars &a, const ConservedVars &b);
ConservedVars operator * (const ConservedVars &a, const ConservedVars &b);
ConservedVars operator / (const ConservedVars &a, const ConservedVars &b);

ConservedVars operator * (real r, const ConservedVars &a);
ConservedVars operator * (const ConservedVars &a, real r);
ConservedVars operator / (const ConservedVars &a, real r);

void Zero        (ConservedVars &CV);
void NonNegative (ConservedVars &CV);
void Exp         (ConservedVars &CV);
void Log         (ConservedVars &CV);

// === XtraVars

XtraVars operator + (const XtraVars &a, const XtraVars &b);
XtraVars operator - (const XtraVars &a, const XtraVars &b);
XtraVars operator * (const XtraVars &a, const XtraVars &b);
XtraVars operator / (const XtraVars &a, const XtraVars &b);

XtraVars operator * (real r, const XtraVars &a);
XtraVars operator * (const XtraVars &a, real r);
XtraVars operator / (const XtraVars &a, real r);

void Zero        (XtraVars &CV);
void NonNegative (XtraVars &CV);
void Exp         (XtraVars &CV);
void Log         (XtraVars &CV);

// === PrimitiveVars

PrimitiveVars operator + (const PrimitiveVars &a, const PrimitiveVars &b);
PrimitiveVars operator - (const PrimitiveVars &a, const PrimitiveVars &b);
PrimitiveVars operator * (const PrimitiveVars &a, const PrimitiveVars &b);
PrimitiveVars operator / (const PrimitiveVars &a, const PrimitiveVars &b);

PrimitiveVars operator * (real r, const PrimitiveVars &a);
PrimitiveVars operator * (const PrimitiveVars &a, real r);
PrimitiveVars operator / (const PrimitiveVars &a, real r);

void Zero (PrimitiveVars &CV);

// === AllVariables

// structure for AllVariables
struct AllVariables {
  ConservedVars  U;
  XtraVars      XU;
  
  AllVariables() {}
  AllVariables(ConservedVars &CV, XtraVars &XV): U(CV), XU(XV) {}
  AllVariables(ConservedVars CV, XtraVars XV): U(CV), XU(XV) {}
};

// friend operators for AllVariables

AllVariables operator + (const AllVariables &a, const AllVariables &b);
AllVariables operator - (const AllVariables &a, const AllVariables &b);
AllVariables operator * (const AllVariables &a, const AllVariables &b);
AllVariables operator / (const AllVariables &a, const AllVariables &b);

AllVariables operator * (const real r, const AllVariables &a);
AllVariables operator * (const AllVariables &a, const real r);
AllVariables operator / (const AllVariables &a, const real r);

#endif
