
/* Euler (Euler equations of gas dynamics)
 * Structures for ConservedVars, XtraVars and PrimitiveVars
 */

#ifndef STRUCTURES_EULER_H
#define STRUCTURES_EULER_H

#include "structures-base.h"

#ifndef GAMMA
  #include "chaos.h"
#endif

// === ConservedVars

struct ConservedVars : ConservedVarsBase {
  real   rho;     // density
  vector m;       // momentum
  real   E;       // energy
  
  real norm() const
  {
    return fabs(rho)+m.norm()+fabs(E);
  }
};

// === XtraVars

struct XtraVars : XtraVarsBase {
  real   p;       // pressure
  vector u;       // velocity
  //real   a;       // entropy
};

// === PrimitiveVars

struct PrimitiveVars : PrimitiveVarsBase {
  real   rho;
  real   p;
  vector u;
  
  // converts PrimitiveVars to ConservedVars (CV)
  void toConservedVars(ConservedVars &v) const {
    v.rho = rho;
    v.m = rho*u;
    #ifndef GAMMA
      real GAMMA = GAMMA_MEAN - GAMMA_DEV + 2 * GAMMA_DEV * chaos.m_uniform[0];
    #endif
    v.E = p/(GAMMA-1) + 0.5*rho*u.sqr();
  }
  
  inline PrimitiveVars() {}
  // picks PrimitiveVars from ConservedVars (CV) and XtraVars (XV)
  inline PrimitiveVars(const ConservedVars &CV, const XtraVars &XV) {
    rho = CV.rho;
    p   = XV.p;
    u   = XV.u;
  }
};

#endif
