
/* MHD (Magneto-HydroDynamics equations)
 * Structures for ConservedVars, XtraVars and PrimitiveVars
 */

#ifndef STRUCTURES_MHD_H
#define STRUCTURES_MHD_H

#include "structures-base.h"

// === ConservedVars

struct ConservedVars : ConservedVarsBase {
  real   rho;  // density
  vector m;    // pressure
  real   E;    // energy
  vector B;    // magnetic field
  
  real norm() const
  {
    return fabs(rho)+m.norm()+fabs(E)+B.norm();
  }
};

// === XtraVars

struct XtraVars : XtraVarsBase {
  real   p;    // pressure
  vector u;    // velocity
  real   a;    // entropy
  vector cf;   // flux
};

// === PrimitiveVars

struct PrimitiveVars : PrimitiveVarsBase {
  real   rho;
  real   p;
  vector u;
  vector B;
  
  // converts PrimitiveVars to ConservedVars (CV)
  void toConservedVars(ConservedVars &v) const {
    v.rho = rho;
    v.m = rho*u;
    v.E = p/(GAMMA-1) + 0.5*rho*u.sqr() + 0.5*B.sqr();
    v.B = B;
  }
  
  inline PrimitiveVars() {}
  // picks PrimitiveVars from ConservedVars (CV) and XtraVars (XV)
  inline PrimitiveVars(const ConservedVars &CV, const XtraVars &XV) {
    rho = CV.rho;
    p   = XV.p;
    u   = XV.u;
    B   = CV.B;
  }
};

#endif
