
/* Wave (Wave equation)
 * Structures for ConservedVars, XtraVars and PrimitiveVars
 */

#ifndef STRUCTURES_WAVE_H
#define STRUCTURES_WAVE_H

#include "structures-base.h"

// === ConservedVars

struct ConservedVars : ConservedVarsBase {
  real   p;     // acoustic pressure
  vector u;     // u_t = (p_x, p_y, p_z)
  
  real norm() const
  {
    return fabs(p) + u.norm();
  }
};

// === XtraVars

struct XtraVars : XtraVarsBase {
  real   E;     // energy
  
  #if ISOTROPIC
  real   c;     // material coefficient (isotropic)
  #else
  matrix c;     // material coefficient (anisotropic)
  #endif
};

// === PrimitiveVars

struct PrimitiveVars : PrimitiveVarsBase {
  real   p;
  vector u;     // u_t = (p_x, p_y, p_z)
  
  // converts PrimitiveVars to ConservedVars (CV)
  void toConservedVars(ConservedVars &CV) const {
    CV.p   = p;
    CV.u   = u;
  }
  
  inline PrimitiveVars() {}
  // picks PrimitiveVars from ConservedVars (CV) and XtraVars (XV)
  inline PrimitiveVars(const ConservedVars &CV, const XtraVars &XV) {
    p   = CV.p;
    u   = CV.u;
  }
};

#endif
