
#ifndef TIME_H
#define TIME_H

// sub-step counter for multi-step integrators
extern int TIME_SUB_STEP;

// === GENERAL TIME routines (defined in equation.cpp)

// advance time step up to time 't + dt' for the 'internal' cells
void time_step_internal_cells ();

// advance time step up to time 't + dt' for the 'external' cells
void time_step_external_cells ();

// sub-step for high order multi-step integrators for the ALL cells
void time_sub_step ();

// sub-step for high order multi-step integrators for the 'internal' cells only
void time_sub_step_internal_cells ();

// sub-step for high order multi-step integrators for the 'boundary' cells only
void time_sub_step_external_cells ();

// computes source term for cell (i,j,k)
void time_compute_source (const int &i, const int &j, const int &k);

// computes flux term in direction d for cell (i,j,k)
void time_compute_flux (const int &i, const int &j, const int &k, const int &d);

// apply computed fluxes and sources to 'U'
// executed at the end of 'time_sub_step()' if COMPUTE_BOUNDARY_CELLS=1
void time_apply_fluxes_and_sources ();

#endif
