
#include "topography-hierarchical.h"

#include "equation.h"
#include "chaos.h"

int HR_MAX = 0;

int  J [HR+1];
real C [HR+1];

void hierarchical_init () {
  
  #if ANTI_ALIASING
  HR_MAX = min( max ( (int) floor ( log2(NXl_FULL) ) + 1, (int) floor ( log2(NYl_FULL) ) ) + 1, HR );
  #else
  HR_MAX = HR;
  #endif
  
  J[0] = 2;
  for (int m=1; m<=HR_MAX; m++)
    J[m] = 1<<(m-1);
  
  C[0] = 0.0;
  for (int m=1; m<=HR_MAX; m++)
    C[m] = pow(2, -HR_DECAY*m);
}

real reference_hat_function (const real &x) {
  if (x <= 0.5)
    return 2*x;
  else
    return 2 - 2*x;
}

bool is_in_support (const real &x, const int &m, const int &j) {
  return ( ((real)j)/J[m] < x ) && ( x < ((real)(j+1))/J[m] );
}

real hr_basis_function (const real &x, const int &m, const int &j) {
  
  // special treatment of the 0-th level
  if (m == 0)
    return j + (1.0 - 2*j) * x;
  
  if (!is_in_support(x,m,j))
    return 0.0;
  
  else
    return reference_hat_function ( J[m] * ( x - ((real)j)/J[m] ) );
}

bool is_in_shifted_support (const real &x, const int &m, const int &j) {
  return ( (j+0.5)/J[m] < x ) && ( x < (j+1.5)/J[m] );
}

real hr_shifted_basis_function (const real &x, const int &m, const int &j) {
  
  if (!is_in_shifted_support(x,m,j))
    return 0.0;
  
  else
    return reference_hat_function ( J[m] * ( x - (j+0.5)/J[m] ) );
}
