
#include "topography.h"

#include "equation.h"
#include "multi.h"
#include "quadrature.h"

real b (const int &i, const int &j) {
  
  // use quadrature rule
  
  real vb = 0.0;
  real qb;
  real x = get_x(i);
  real y = get_y(j);
  
  for (int px=0; px<(HAVE_DIR(X) ? QUAD_NUM : 1); px++)
    for (int py=0; py<(HAVE_DIR(Y) ? QUAD_NUM : 1); py++) {
      qb = b ( x + QUAD_POINTS[px] * dx[X],
               y + QUAD_POINTS[py] * dx[Y] );
      if (HAVE_DIR(X)) qb *= QUAD_WEIGHTS[px];
      if (HAVE_DIR(Y)) qb *= QUAD_WEIGHTS[py];
      vb += qb;
    }
  
  return vb;
}

#if STANDARD_TOPOGRAPHY_QUAD

// 2nd order accurate source quadrature
void compute_source (const int &i, const int &j) {
  Source[i][j][0].hu[X] = - GRAVITY * U[i][j][0].h * ( Xtra[i+1][j][0].b - Xtra[i-1][j][0].b ) / (2*dx[X]);
  if (HAVE_DIR(Y))
    Source[i][j][0].hu[Y] = - GRAVITY * U[i][j][0].h * ( Xtra[i][j+1][0].b - Xtra[i][j-1][0].b ) / (2*dx[Y]);
}

#endif
