
#include "topography.h"

#include "topography-hierarchical.h"
#include "chaos.h"

///::title:: Hierarchical hat basis (1D)
///::vars:: HR=9 ANTI_ALIASING=1
///::consts:: HR_DECAY=1.5
///::depends:: topography-hierarchical
///::macro:: LEVEL_INIT_HOOK topography-hierarchical.h hierarchical_init();

void topography_set_chaos_size () {
  // HR_SAMPLES = min(HR, HR_MAX + TYPE) needs to be used instead of HR_MAX
  // to ensure coupling of TYPE 0 and TYPE 1 samples
  int HR_SAMPLES = min (HR, HR_MAX + TYPE);
  CHAOS_SIZE [TOPOGRAPHY] [GLOBAL] [UNIFORM] = (1<<HR_SAMPLES)-1;
  CHAOS_SIZE [TOPOGRAPHY] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [TOPOGRAPHY] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [TOPOGRAPHY] [LOCAL]  [NORMAL]  = 0;
}

real b (real x, real y) {
  
  real value = 0.0;
  long_int counter = 0;
  
  // mean for bottom topography
  if (HR_MAX >= 3) value += 1.0 * hr_basis_function(x/MAXX,3,1);
  if (HR_MAX >= 4) value -= 0.8 * hr_basis_function(x/MAXX,4,4);
  if (HR_MAX >= 5) value += 0.3 * hr_basis_function(x/MAXX,5,10);
  
  // uncertainty perturbations
  for (int m=1; m<=HR_MAX; m++)
    for (int j=0; j<J[m]; j++)
      value += C[m] * hr_basis_function(x/MAXX,m,j) * (2 * chaos [TOPOGRAPHY] [GLOBAL] [UNIFORM] [counter++] - 1);
  
  return value;
}
