
#include "equation.h"
#include "chaos.h"

///::title:: Spherical Blast Waves in a strong field (3D)
///::vars:: NX=64 NY=96 NZ=32 MAXT=1.0
///::consts:: MAXX=1 MAXY=1.5 MAXZ=0.5
///::consts:: N_UNIFORM=8 N_NORMAL=0
///::bc:: REPEAT

void model_set_chaos_size () {
  CHAOS_SIZE [MODEL] [GLOBAL] [UNIFORM] = 8;
  CHAOS_SIZE [MODEL] [GLOBAL] [NORMAL]  = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [UNIFORM] = 0;
  CHAOS_SIZE [MODEL] [LOCAL]  [NORMAL]  = 0;
}

void initial_data (PrimitiveVars &v, real x, real y, real z)
{
  v.rho = 1.0;
  
  real r = sqr(x-0.5) + sqr(y-0.75) + sqr(z-0.25);
  real phi = (x-0.5) / sqrt(r);
  real xi  = (y-0.75)  / sqrt(r);
  real r_max = sqr( 0.1 + 0.02 * chaos [MODEL] [GLOBAL] [UNIFORM] [0] * sin(phi) + 0.01 * chaos [MODEL] [GLOBAL] [UNIFORM] [1] * sin(10*xi) );
  
  if ( r < 0.01 ) {
    v.p = 10.0;
    v.p += 0.5 * chaos [MODEL] [GLOBAL] [UNIFORM] [2];
    v.p += 1.0 * chaos [MODEL] [GLOBAL] [UNIFORM] [3] * sin(4*(x-0.25));
    v.p += 0.5 * chaos [MODEL] [GLOBAL] [UNIFORM] [4] * cos(8*(y-0.5));
    v.p += 1.0 * chaos [MODEL] [GLOBAL] [UNIFORM] [5] * sin(6*(z-0.5));
  }
  else
    v.p = 0.1;
  
  v.B[X] = sqrt(2*pi) + 0.2 * chaos [MODEL] [GLOBAL] [UNIFORM] [6];
  v.B[Y] = sqrt(2*pi) + 0.2 * chaos [MODEL] [GLOBAL] [UNIFORM] [7];
}
